import os
import tempfile
import unittest
from xml.dom import minidom

from avocado import Test
from avocado.core import job
from avocado.core.result import Result
from avocado.plugins import xunit

from .. import setup_avocado_loggers, temp_dir_prefix

try:
    import xmlschema
    SCHEMA_CAPABLE = True
except ImportError:
    SCHEMA_CAPABLE = False


setup_avocado_loggers()


UNIQUE_ID = '0000000000000000000000000000000000000000'
LOGFILE = None


class ParseXMLError(Exception):
    pass


class xUnitSucceedTest(unittest.TestCase):

    def setUp(self):

        class SimpleTest(Test):

            def test(self):
                pass

        self.tmpfile = tempfile.mkstemp()
        prefix = temp_dir_prefix(__name__, self, 'setUp')
        self.tmpdir = tempfile.TemporaryDirectory(prefix=prefix)
        config = {'job.run.result.xunit.output': self.tmpfile[1],
                  'run.results_dir': self.tmpdir.name,
                  'run.keep_tmp': True}
        self.job = job.Job(config)
        self.job.setup()
        self.test_result = Result(UNIQUE_ID, LOGFILE)
        self.test_result.tests_total = 1
        self.test_result.logfile = ("/.../avocado/job-results/"
                                    "job-2018-11-28T16.27-8fef221/job.log")
        self.test1 = SimpleTest(config=self.job.config, base_logdir=self.tmpdir.name)
        self.test1._Test__status = 'PASS'
        self.test1._Test__logfile = ''
        self.test1.time_elapsed = 678.23689

    def tearDown(self):
        self.job.cleanup()
        errs = []
        cleanups = (lambda: os.close(self.tmpfile[0]),
                    lambda: os.remove(self.tmpfile[1]),
                    self.tmpdir.cleanup)
        for cleanup in cleanups:
            try:
                cleanup()
            except Exception as exc:
                errs.append(str(exc))
        self.assertFalse(errs, "Failures occurred during cleanup:\n%s"
                         % "\n".join(errs))

    @unittest.skipUnless(SCHEMA_CAPABLE,
                         'Unable to validate schema due to missing xmlschema library')
    def test_add_success(self):
        self.test_result.start_test(self.test1)
        self.test_result.end_test(self.test1.get_state())
        self.test_result.end_tests()
        xunit_result = xunit.XUnitResult()
        xunit_result.render(self.test_result, self.job)
        xunit_output = self.job.config.get('job.run.result.xunit.output')
        with open(xunit_output, 'rb') as fp:
            xml = fp.read()
        try:
            dom = minidom.parseString(xml)
        except Exception as details:
            raise ParseXMLError("Error parsing XML: '%s'.\nXML Contents:\n%s" % (details, xml))
        self.assertTrue(dom)

        els = dom.getElementsByTagName('testsuite')
        self.assertEqual(len(els), 1)
        self.assertEqual(els[0].attributes['time'].value, '678.237')

        els = dom.getElementsByTagName('testcase')
        self.assertEqual(len(els), 1)
        self.assertEqual(els[0].attributes['time'].value, '678.237')

        junit_xsd = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                 os.path.pardir, ".data",
                                                 'jenkins-junit.xsd'))
        xml_schema = xmlschema.XMLSchema(junit_xsd)
        self.assertTrue(xml_schema.is_valid(xunit_output))

    def test_max_test_log_size(self):
        def get_system_out(out):
            return out[out.find(b"<system-out>"):out.find(b"<system-out/>")]
        log = tempfile.NamedTemporaryFile(dir=self.tmpdir.name, delete=False)
        log_content = b"1234567890" * 100
        log_content += b"this should not be present" + b"0987654321" * 100
        log.write(log_content)
        log_path = log.name
        log.close()
        self.test1._Test__status = "ERROR"
        self.test1._Test__logfile = log_path
        self.test_result.start_test(self.test1)
        self.test_result.end_test(self.test1.get_state())
        self.test_result.end_tests()
        xunit_result = xunit.XUnitResult()
        # setting the default value
        self.job.config['job.run.result.xunit.max_test_log_chars'] = 100000
        xunit_result.render(self.test_result, self.job)
        xunit_output = self.job.config.get('job.run.result.xunit.output')
        with open(xunit_output, 'rb') as fp:
            unlimited = fp.read()
        # setting a small value
        self.job.config['job.run.result.xunit.max_test_log_chars'] = 10
        xunit_result.render(self.test_result, self.job)
        with open(xunit_output, 'rb') as fp:
            limited = fp.read()
        # back to the default value
        self.job.config['job.run.result.xunit.max_test_log_chars'] = 100000
        xunit_result.render(self.test_result, self.job)
        with open(xunit_output, 'rb') as fp:
            limited_but_fits = fp.read()
        self.assertLess(len(limited), len(unlimited) - 500,
                        "Length of xunit limitted to 10 chars was greater "
                        "than (unlimited - 500). Unlimited output:\n%s\n\n"
                        "Limited output:\n%s" % (unlimited, limited))
        unlimited_output = get_system_out(unlimited)
        self.assertIn(log_content, unlimited_output)
        self.assertEqual(unlimited_output, get_system_out(limited_but_fits))
        self.assertIn(b"this should not be present", unlimited)
        self.assertNotIn(b"this should not be present", limited)
        self.assertIn(b"1234567890", unlimited)
        self.assertNotIn(b"1234567890", limited)
        self.assertIn(b"12345", limited)
        self.assertIn(b"0987654321", unlimited)
        self.assertNotIn(b"0987654321", limited)
        self.assertIn(b"54321", limited)


if __name__ == '__main__':
    unittest.main()



import io
import socket
import unittest

from avocado.utils import datadrainer


class Base(unittest.TestCase):

    def test_instantiate(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            datadrainer.BaseDrainer(None)


class Magic(datadrainer.BaseDrainer):

    name = 'test_utils_datadrainer.Magic'
    magic = 'MAGIC_magic_MAGIC'

    def data_available(self):
        return True

    def read(self):
        return self.magic

    def write(self, data):
        self.destination = data  # pylint: disable=W0201
        self._internal_quit = True


class Custom(unittest.TestCase):

    def test(self):
        magic = Magic(None)
        magic.start()
        magic.wait()
        self.assertEqual(Magic.magic, magic.destination)


class Socket(datadrainer.FDDrainer):

    name = 'test_utils_datadrainer.Socket'

    def __init__(self, source):
        super(Socket, self).__init__(source)
        self.data_buffer = io.BytesIO()
        self._write_count = 0

    def write(self, data):
        self.data_buffer.write(data)
        self._write_count += len(data)
        if self._write_count > 2:
            self._internal_quit = True


class CustomSocket(unittest.TestCase):

    def setUp(self):
        self.socket1, self.socket2 = socket.socketpair(socket.AF_UNIX)

    def test(self):
        socket_drainer = Socket(self.socket2.fileno())
        socket_drainer.start()
        self.socket1.send(b'1')
        self.socket1.send(b'2')
        self.socket1.send(b'3')
        socket_drainer.wait()
        self.assertEqual(socket_drainer.data_buffer.getvalue(), b'123')

    def tearDown(self):
        self.socket1.close()
        self.socket2.close()


class SocketBuffer(datadrainer.BufferFDDrainer):

    name = 'test_utils_datadrainer.SocketBuffer'

    def __init__(self, source):
        super(SocketBuffer, self).__init__(source)
        self._stop_check = lambda: len(self.data) > 2


class CustomSocketBuffer(unittest.TestCase):

    def setUp(self):
        self.socket1, self.socket2 = socket.socketpair(socket.AF_UNIX)

    def test(self):
        socket_drainer = SocketBuffer(self.socket2.fileno())
        socket_drainer.start()
        self.socket1.send(b'1')
        self.socket1.send(b'2')
        self.socket1.send(b'3')
        socket_drainer.wait()
        self.assertEqual(socket_drainer.data, b'123')

    def tearDown(self):
        self.socket1.close()
        self.socket2.close()



import os
import sys
import unittest

from avocado.utils import process, script

from .. import BASEDIR, TestCaseTmpDir

UNITTEST_GOOD = """from avocado import Test
from unittest import main


class CustomBaselogDirInit(Test):
    def __init__(self, *args, **kwargs):
        super(CustomBaselogDirInit, self).__init__(base_logdir="%s", *args,
                                                   **kwargs)

class AvocadoPassTest(CustomBaselogDirInit):
    def test(self):
        self.assertTrue(True)
if __name__ == '__main__':
    main()
"""

UNITTEST_FAIL = """from avocado import Test
from unittest import main


class CustomBaselogDirInit(Test):
    def __init__(self, *args, **kwargs):
        super(CustomBaselogDirInit, self).__init__(base_logdir="%s", *args,
                                                   **kwargs)

class AvocadoFailTest(CustomBaselogDirInit):
    def test(self):
        self.fail('This test is supposed to fail')
if __name__ == '__main__':
    main()
"""

UNITTEST_ERROR = """from avocado import Test
from unittest import main


class CustomBaselogDirInit(Test):
    def __init__(self, *args, **kwargs):
        super(CustomBaselogDirInit, self).__init__(base_logdir="%s", *args,
                                                   **kwargs)

class AvocadoErrorTest(CustomBaselogDirInit):
    def test(self):
        self.error('This test is supposed to error')
if __name__ == '__main__':
    main()
"""


class UnittestCompat(TestCaseTmpDir):

    def setUp(self):
        super(UnittestCompat, self).setUp()
        self.original_pypath = os.environ.get('PYTHONPATH')
        if self.original_pypath is not None:
            os.environ['PYTHONPATH'] = '%s:%s' % (
                BASEDIR, self.original_pypath)
        else:
            os.environ['PYTHONPATH'] = '%s' % BASEDIR
        self.unittest_script_good = script.TemporaryScript(
            'unittest_good.py',
            UNITTEST_GOOD % self.tmpdir.name,
            'avocado_as_unittest_functional')
        self.unittest_script_good.save()
        self.unittest_script_fail = script.TemporaryScript(
            'unittest_fail.py',
            UNITTEST_FAIL % self.tmpdir.name,
            'avocado_as_unittest_functional')
        self.unittest_script_fail.save()
        self.unittest_script_error = script.TemporaryScript(
            'unittest_error.py',
            UNITTEST_ERROR % self.tmpdir.name,
            'avocado_as_unittest_functional')
        self.unittest_script_error.save()

    def test_run_pass(self):
        cmd_line = '%s %s' % (sys.executable, self.unittest_script_good)
        result = process.run(cmd_line)
        self.assertEqual(0, result.exit_status)
        self.assertIn(b'Ran 1 test in', result.stderr)

    def test_run_fail(self):
        cmd_line = '%s %s' % (sys.executable, self.unittest_script_fail)
        result = process.run(cmd_line, ignore_status=True)
        self.assertEqual(1, result.exit_status)
        self.assertIn(b'Ran 1 test in', result.stderr)
        self.assertIn(b'This test is supposed to fail', result.stderr)
        self.assertIn(b'FAILED (failures=1)', result.stderr)

    def test_run_error(self):
        cmd_line = '%s %s' % (sys.executable, self.unittest_script_error)
        result = process.run(cmd_line, ignore_status=True)
        self.assertEqual(1, result.exit_status)
        self.assertIn(b'Ran 1 test in', result.stderr)
        self.assertIn(b'This test is supposed to error', result.stderr)
        self.assertIn(b'FAILED (errors=1)', result.stderr)

    def tearDown(self):
        super(UnittestCompat, self).tearDown()
        self.unittest_script_error.remove()
        self.unittest_script_fail.remove()
        self.unittest_script_good.remove()


if __name__ == '__main__':
    unittest.main()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

"""
Functions dedicated to find and run external commands.
"""

import contextlib
import errno
import fnmatch
import glob
import logging
import os
import re
import select
import shlex
import signal
import subprocess
import threading
import time
from io import BytesIO, UnsupportedOperation

from . import astring, path
from .wait import wait_for

log = logging.getLogger('avocado.test')
stdout_log = logging.getLogger('avocado.test.stdout')
stderr_log = logging.getLogger('avocado.test.stderr')
output_log = logging.getLogger('avocado.test.output')

#: The active wrapper utility script.
CURRENT_WRAPPER = None

#: The global wrapper.
#: If set, run every process under this wrapper.
WRAP_PROCESS = None

#: Set wrapper per program names.
#: A list of wrappers and program names.
#: Format: [ ('/path/to/wrapper.sh', 'progname'), ... ]
WRAP_PROCESS_NAMES_EXPR = []

#: Exception to be raised when users of this API need to know that the
#: execution of a given process resulted in undefined behavior. One
#: concrete example when a user, in an interactive session, let the
#: inferior process exit before before avocado resumed the debugger
#: session. Since the information is unknown, and the behavior is
#: undefined, this situation will be flagged by an exception.
UNDEFINED_BEHAVIOR_EXCEPTION = None

#: The current output record mode.  It's not possible to record
#: both the 'stdout' and 'stderr' streams, and at the same time
#: in the right order, the combined 'output' stream.  So this
#: setting defines the mode.
OUTPUT_CHECK_RECORD_MODE = None

# variable=value bash assignment
_RE_BASH_SET_VARIABLE = re.compile(r"[a-zA-Z]\w*=.*")


class CmdError(Exception):

    def __init__(self, command=None, result=None, additional_text=None):  # pylint: disable=W0231
        self.command = command
        self.result = result
        self.additional_text = additional_text

    def __str__(self):
        return ("Command '%s' failed.\nstdout: %r\nstderr: %r\nadditional_info: %s" %
                (self.command, self.result.stdout, self.result.stderr, self.additional_text))


def can_sudo(cmd=None):
    """
    Check whether sudo is available (or running as root)

    :param cmd: unicode string with the commands
    """
    if os.getuid() == 0:    # Root
        return True

    try:    # Does sudo binary exists?
        path.find_command('sudo')
    except path.CmdNotFoundError:
        return False

    try:
        if cmd:     # Am I able to run the cmd or plain sudo id?
            return not system(cmd, ignore_status=True, sudo=True)
        elif system_output("id -u", ignore_status=True, sudo=True).strip() == "0":
            return True
        else:
            return False
    except OSError:     # Broken sudo binary
        return False


def pid_exists(pid):
    """
    Return True if a given PID exists.

    :param pid: Process ID number.
    """
    try:
        os.kill(pid, 0)
    except OSError as detail:
        if detail.errno == errno.ESRCH:
            return False
    return True


def safe_kill(pid, signal):  # pylint: disable=W0621
    """
    Attempt to send a signal to a given process that may or may not exist.

    :param signal: Signal number.
    """
    if get_owner_id(int(pid)) == 0:
        kill_cmd = 'kill -%d %d' % (int(signal), int(pid))
        try:
            run(kill_cmd, sudo=True)
            return True
        except CmdError:
            return False

    try:
        os.kill(pid, signal)
        return True
    except Exception:  # pylint: disable=W0703
        return False


def get_parent_pid(pid):
    """
    Returns the parent PID for the given process

    :note: This is currently Linux specific.

    :param pid: The PID of child process
    :returns: The parent PID
    :rtype: int
    """
    with open('/proc/%d/stat' % pid, 'rb') as proc_stat:
        parent_pid = proc_stat.read().split(b' ')[-49]
        return int(parent_pid)


def _get_pid_from_proc_pid_stat(proc_path):
    match = re.match(r'\/proc\/([0-9]+)\/.*', proc_path)
    if match is not None:
        return int(match.group(1))


def get_children_pids(parent_pid, recursive=False):
    """
    Returns the children PIDs for the given process

    :note: This is currently Linux specific.

    :param parent_pid: The PID of parent child process
    :returns: The PIDs for the children processes
    :rtype: list of int
    """
    proc_stats = glob.glob('/proc/[123456789]*/stat')
    children = []
    for proc_stat in proc_stats:
        try:
            with open(proc_stat, 'rb') as proc_stat_fp:
                this_parent_pid = int(proc_stat_fp.read().split(b' ')[-49])
        except IOError:
            continue

        if this_parent_pid == parent_pid:
            children.append(_get_pid_from_proc_pid_stat(proc_stat))

    if recursive:
        for child in children:
            children.extend(get_children_pids(child))
    return children


def kill_process_tree(pid, sig=None, send_sigcont=True, timeout=0):
    """
    Signal a process and all of its children.

    If the process does not exist -- return.

    :param pid: The pid of the process to signal.
    :param sig: The signal to send to the processes, defaults to
                :data:`signal.SIGKILL`
    :param send_sigcont: Send SIGCONT to allow killing stopped processes
    :param timeout: How long to wait for the pid(s) to die
                    (negative=infinity, 0=don't wait,
                    positive=number_of_seconds)
    :return: list of all PIDs we sent signal to
    :rtype: list
    """
    def _all_pids_dead(killed_pids):
        for pid in killed_pids:
            if pid_exists(pid):
                return False
        return True

    if sig is None:
        sig = signal.SIGKILL

    if timeout > 0:
        start = time.time()

    if not safe_kill(pid, signal.SIGSTOP):
        return [pid]
    killed_pids = [pid]
    for child in get_children_pids(pid):
        killed_pids.extend(kill_process_tree(int(child), sig, False))
    safe_kill(pid, sig)
    if send_sigcont:
        for pid in killed_pids:
            safe_kill(pid, signal.SIGCONT)
    if timeout == 0:
        return killed_pids
    elif timeout > 0:
        if not wait_for(_all_pids_dead, timeout + start - time.time(),
                        step=0.01, args=(killed_pids[::-1],)):
            raise RuntimeError("Timeout reached when waiting for pid %s "
                               "and children to die (%s)" % (pid, timeout))
    else:
        while not _all_pids_dead(killed_pids[::-1]):
            time.sleep(0.01)
    return killed_pids


def kill_process_by_pattern(pattern):
    """
    Send SIGTERM signal to a process with matched pattern.

    :param pattern: normally only matched against the process name
    """
    cmd = "pkill -f %s" % pattern
    result = run(cmd, ignore_status=True)
    if result.exit_status:
        logging.error("Failed to run '%s': %s", cmd, result)
    else:
        logging.info("Succeed to run '%s'.", cmd)


def process_in_ptree_is_defunct(ppid):
    """
    Verify if any processes deriving from PPID are in the defunct state.

    Attempt to verify if parent process and any children from PPID is defunct
    (zombie) or not.

    :param ppid: The parent PID of the process to verify.
    """
    # TODO: This relies on the GNU version of ps (need to fix MacOS support)
    defunct = False
    try:
        pids = get_children_pids(ppid)
    except CmdError:  # Process doesn't exist
        return True
    for pid in pids:
        cmd = "ps --no-headers -o cmd %d" % int(pid)
        proc_name = system_output(cmd, ignore_status=True, verbose=False)
        if '<defunct>' in proc_name:
            defunct = True
            break
    return defunct


def binary_from_shell_cmd(cmd):
    """
    Tries to find the first binary path from a simple shell-like command.

    :note: It's a naive implementation, but for commands like:
           `VAR=VAL binary -args || true` gives the right result (binary)
    :param cmd: simple shell-like binary
    :type cmd: unicode string
    :return: first found binary from the cmd
    """
    cmds = shlex.split(cmd)
    for item in cmds:
        if not _RE_BASH_SET_VARIABLE.match(item):
            return item
    raise ValueError("Unable to parse first binary from '%s'" % cmd)


#: This is kept for compatibility purposes, but is now deprecated and
#: will be removed in later versions.  Please use :func:`shlex.split`
#: instead.
cmd_split = shlex.split


class CmdResult:

    """
    Command execution result.

    :param command: the command line itself
    :type command: str
    :param exit_status: exit code of the process
    :type exit_status: int
    :param stdout: content of the process stdout
    :type stdout: bytes
    :param stderr: content of the process stderr
    :type stderr: bytes
    :param duration: elapsed wall clock time running the process
    :type duration: float
    :param pid: ID of the process
    :type pid: int
    :param encoding: the encoding to use for the text version
                     of stdout and stderr, by default
                     :data:`avocado.utils.astring.ENCODING`
    :type encoding: str
    """

    def __init__(self, command="", stdout=b"", stderr=b"",
                 exit_status=None, duration=0, pid=None,
                 encoding=None):
        self.command = command
        self.exit_status = exit_status
        #: The raw stdout (bytes)
        self.stdout = stdout
        #: The raw stderr (bytes)
        self.stderr = stderr
        self.duration = duration
        self.interrupted = False
        self.pid = pid
        if encoding is None:
            encoding = astring.ENCODING
        self.encoding = encoding

    def __str__(self):
        return '\n'.join("%s: %r" % (key, getattr(self, key, "MISSING"))
                         for key in ('command', 'exit_status', 'duration',
                                     'interrupted', 'pid', 'encoding',
                                     'stdout', 'stderr'))

    @property
    def stdout_text(self):
        if hasattr(self.stdout, 'decode'):
            return self.stdout.decode(self.encoding)
        if isinstance(self.stdout, str):
            return self.stdout
        raise TypeError("Unable to decode stdout into a string-like type")

    @property
    def stderr_text(self):
        if hasattr(self.stderr, 'decode'):
            return self.stderr.decode(self.encoding)
        if isinstance(self.stderr, str):
            return self.stderr
        raise TypeError("Unable to decode stderr into a string-like type")


class FDDrainer:

    def __init__(self, fd, result, name=None, logger=None, logger_prefix='%s',
                 stream_logger=None, ignore_bg_processes=False, verbose=False):
        """
        Reads data from a file descriptor in a thread, storing locally in
        a file-like :attr:`data` object.

        :param fd: a file descriptor that will be read (drained) from
        :type fd: int
        :param result: a :class:`CmdResult` instance associated with the process
                       used to detect if the process is still running and
                       if there's still data to be read.
        :type result: a :class:`CmdResult` instance
        :param name: a descriptive name that will be passed to the Thread name
        :type name: str
        :param logger: the logger that will be used to (interactively) write
                       the content from the file descriptor
        :type logger: :class:`logging.Logger`
        :param logger_prefix: the prefix used when logging the data
        :type logger_prefix: str with one %-style string formatter
        :param ignore_bg_processes: When True the process does not wait for
                    child processes which keep opened stdout/stderr streams
                    after the main process finishes (eg. forked daemon which
                    did not closed the stdout/stderr). Note this might result
                    in missing output produced by those daemons after the
                    main thread finishes and also it allows those daemons
                    to be running after the process finishes.
        :type ignore_bg_processes: boolean
        :param verbose: whether to log in both the logger and stream_logger
        :type verbose: boolean
        """
        self.fd = fd
        self.name = name
        self.data = BytesIO()
        # TODO: check if, when the process finishes, the FD doesn't
        # automatically close.  This may be used as the detection
        # instead.
        self._result = result
        self._thread = None
        self._logger = logger
        self._logger_prefix = logger_prefix
        self._stream_logger = stream_logger
        self._ignore_bg_processes = ignore_bg_processes
        self._verbose = verbose

    def _log_line(self, line, newline_for_stream='\n'):
        line = astring.to_text(line, self._result.encoding,
                               'replace')
        if self._logger is not None:
            self._logger.debug(self._logger_prefix, line)
        if self._stream_logger is not None:
            self._stream_logger.debug(line + newline_for_stream)

    def _drainer(self):
        """
        Read from fd, storing and optionally logging the output
        """
        bfr = b''
        while True:
            if self._ignore_bg_processes:
                has_io = select.select([self.fd], [], [], 1)[0]
                if (not has_io and self._result.exit_status is not None):
                    # Exit if no new data and main process has finished
                    break
                if not has_io:
                    # Don't read unless there are new data available
                    continue
            tmp = os.read(self.fd, 8192)
            if not tmp:
                break
            self.data.write(tmp)
            if self._verbose:
                bfr += tmp
                lines = bfr.splitlines()
                for line in lines[:-1]:
                    self._log_line(line)
                if bfr.endswith(b'\n'):
                    self._log_line(lines[-1])
                else:
                    self._log_line(lines[-1], '')
                bfr = b''

    def start(self):
        self._thread = threading.Thread(target=self._drainer, name=self.name)
        self._thread.daemon = True
        self._thread.start()

    def flush(self):
        self._thread.join()
        if self._stream_logger is not None:
            for handler in self._stream_logger.handlers:
                # FileHandler has a close() method, which we expect will
                # flush the file on disk.  SocketHandler, MemoryHandler
                # and other logging handlers (custom ones?) also have
                # the same interface, so let's try to use it if available
                stream = getattr(handler, 'stream', None)
                if (stream is not None) and (not stream.closed):
                    if hasattr(stream, 'fileno'):
                        try:
                            fileno = stream.fileno()
                            os.fsync(fileno)
                        except UnsupportedOperation:
                            pass
                if hasattr(handler, 'close'):
                    handler.close()


class SubProcess:

    """
    Run a subprocess in the background, collecting stdout/stderr streams.
    """

    def __init__(self, cmd, verbose=True, allow_output_check=None,
                 shell=False, env=None, sudo=False,
                 ignore_bg_processes=False, encoding=None):
        """
        Creates the subprocess object, stdout/err, reader threads and locks.

        :param cmd: Command line to run.
        :type cmd: str
        :param verbose: Whether to log the command run and stdout/stderr.
        :type verbose: bool
        :param allow_output_check: Whether to record the output from this
                                   process (from stdout and stderr) in the
                                   test's output record files. Valid values:
                                   'stdout', for standard output *only*,
                                   'stderr' for standard error *only*,
                                   'both' for both standard output and error
                                   in separate files, 'combined' for
                                   standard output and error in a single file,
                                   and 'none' to disable all recording. 'all'
                                   is also a valid, but deprecated, option that
                                   is a synonym of 'both'.  If an explicit value
                                   is not given to this parameter, that is, if
                                   None is given, it defaults to using the module
                                   level configuration, as set by
                                   :data:`OUTPUT_CHECK_RECORD_MODE`.  If the
                                   module level configuration itself is not set,
                                   it defaults to 'none'.
        :type allow_output_check: str
        :param shell: Whether to run the subprocess in a subshell.
        :type shell: bool
        :param env: Use extra environment variables.
        :type env: dict
        :param sudo: Whether the command requires admin privileges to run,
                     so that sudo will be prepended to the command.
                     The assumption here is that the user running the command
                     has a sudo configuration such that a password won't be
                     prompted. If that's not the case, the command will
                     straight out fail.
        :type sudo: bool
        :param ignore_bg_processes: When True the process does not wait for
                    child processes which keep opened stdout/stderr streams
                    after the main process finishes (eg. forked daemon which
                    did not closed the stdout/stderr). Note this might result
                    in missing output produced by those daemons after the
                    main thread finishes and also it allows those daemons
                    to be running after the process finishes.
        :param encoding: the encoding to use for the text representation
                         of the command result stdout and stderr, by default
                         :data:`avocado.utils.astring.ENCODING`
        :type encoding: str
        :raises: ValueError if incorrect values are given to parameters
        """
        if encoding is None:
            encoding = astring.ENCODING
        if sudo:
            self.cmd = self._prepend_sudo(cmd, shell)
        else:
            self.cmd = cmd
        self.verbose = verbose
        if allow_output_check is None:
            allow_output_check = OUTPUT_CHECK_RECORD_MODE
        if allow_output_check is None:
            allow_output_check = 'both'
        if allow_output_check not in ('stdout', 'stderr', 'both',
                                      'combined', 'none', 'all'):
            msg = ("Invalid value (%s) set in allow_output_check" %
                   allow_output_check)
            raise ValueError(msg)
        self.allow_output_check = allow_output_check
        self.result = CmdResult(self.cmd, encoding=encoding)
        self.shell = shell
        if env:
            self.env = os.environ.copy()
            self.env.update(env)
        else:
            self.env = None
        self._popen = None

        # Drainers used when reading from the PIPEs and writing to
        # files and logs
        self._stdout_drainer = None
        self._stderr_drainer = None
        self._combined_drainer = None

        self._ignore_bg_processes = ignore_bg_processes

    def __repr__(self):
        if self._popen is None:
            rc = '(not started)'
        elif self.result.exit_status is None:
            rc = '(running)'
        else:
            rc = self.result.exit_status
        return '%s(cmd=%r, rc=%r)' % (self.__class__.__name__, self.cmd, rc)

    def __str__(self):
        if self._popen is None:
            rc = '(not started)'
        elif self.result.exit_status is None:
            rc = '(running)'
        else:
            rc = '(finished with exit status=%d)' % self.result.exit_status
        return '%s %s' % (self.cmd, rc)

    @staticmethod
    def _prepend_sudo(cmd, shell):
        if os.getuid() != 0:
            try:
                sudo_cmd = '%s -n' % path.find_command('sudo', check_exec=False)
            except path.CmdNotFoundError as details:
                log.error(details)
                log.error('Parameter sudo=True provided, but sudo was '
                          'not found. Please consider adding sudo to '
                          'your OS image')
                return cmd
            if shell:
                if ' -s' not in sudo_cmd:
                    sudo_cmd = '%s -s' % sudo_cmd
            cmd = '%s %s' % (sudo_cmd, cmd)
        return cmd

    def _init_subprocess(self):
        if self._popen is None:
            if self.verbose:
                log.info("Running '%s'", self.cmd)
            if self.shell is False:
                cmd = shlex.split(self.cmd)
            else:
                cmd = self.cmd
            try:
                if self.allow_output_check == 'combined':
                    stderr = subprocess.STDOUT
                else:
                    stderr = subprocess.PIPE
                self._popen = subprocess.Popen(cmd,
                                               stdout=subprocess.PIPE,
                                               stderr=stderr,
                                               shell=self.shell,
                                               env=self.env)
            except OSError as details:
                details.strerror += " (%s)" % self.cmd
                raise details

            self.start_time = time.time()  # pylint: disable=W0201

            # prepare fd drainers
            if self.allow_output_check == 'combined':
                self._combined_drainer = FDDrainer(
                    self._popen.stdout.fileno(),
                    self.result,
                    name="%s-combined" % self.cmd,
                    logger=log,
                    logger_prefix="[output] %s",
                    # FIXME, in fact, a new log has to be used here
                    stream_logger=output_log,
                    ignore_bg_processes=self._ignore_bg_processes,
                    verbose=self.verbose)
                self._combined_drainer.start()

            else:
                if self.allow_output_check == 'none':
                    stdout_stream_logger = None
                    stderr_stream_logger = None
                else:
                    stdout_stream_logger = stdout_log
                    stderr_stream_logger = stderr_log
                self._stdout_drainer = FDDrainer(
                    self._popen.stdout.fileno(),
                    self.result,
                    name="%s-stdout" % self.cmd,
                    logger=log,
                    logger_prefix="[stdout] %s",
                    stream_logger=stdout_stream_logger,
                    ignore_bg_processes=self._ignore_bg_processes,
                    verbose=self.verbose)
                self._stderr_drainer = FDDrainer(
                    self._popen.stderr.fileno(),
                    self.result,
                    name="%s-stderr" % self.cmd,
                    logger=log,
                    logger_prefix="[stderr] %s",
                    stream_logger=stderr_stream_logger,
                    ignore_bg_processes=self._ignore_bg_processes,
                    verbose=self.verbose)

                # start stdout/stderr threads
                self._stdout_drainer.start()
                self._stderr_drainer.start()

            def signal_handler(signum, frame):  # pylint: disable=W0613
                self.result.interrupted = "signal/ctrl+c"
                self.wait()
                signal.default_int_handler()
            try:
                signal.signal(signal.SIGINT, signal_handler)
            except ValueError:
                if self.verbose:
                    log.info("Command %s running on a thread", self.cmd)

    def _fill_results(self, rc):
        self._init_subprocess()
        self.result.exit_status = rc
        if self.result.duration == 0:
            self.result.duration = time.time() - self.start_time
        if self.verbose:
            log.info("Command '%s' finished with %s after %ss", self.cmd, rc,
                     self.result.duration)
        self.result.pid = self._popen.pid
        self._fill_streams()

    def _fill_streams(self):
        """
        Close subprocess stdout and stderr, and put values into result obj.
        """
        # Cleaning up threads
        if self._combined_drainer is not None:
            self._combined_drainer.flush()
        if self._stdout_drainer is not None:
            self._stdout_drainer.flush()
        if self._stderr_drainer is not None:
            self._stderr_drainer.flush()
        # Clean subprocess pipes and populate stdout/err
        self.result.stdout = self.get_stdout()
        self.result.stderr = self.get_stderr()

    def start(self):
        """
        Start running the subprocess.

        This method is particularly useful for background processes, since
        you can start the subprocess and not block your test flow.

        :return: Subprocess PID.
        :rtype: int
        """
        self._init_subprocess()
        return self._popen.pid

    def get_stdout(self):
        """
        Get the full stdout of the subprocess so far.

        :return: Standard output of the process.
        :rtype: str
        """
        self._init_subprocess()
        if self._combined_drainer is not None:
            return self._combined_drainer.data.getvalue()
        return self._stdout_drainer.data.getvalue()

    def get_stderr(self):
        """
        Get the full stderr of the subprocess so far.

        :return: Standard error of the process.
        :rtype: str
        """
        self._init_subprocess()
        if self._combined_drainer is not None:
            return ''
        return self._stderr_drainer.data.getvalue()

    def terminate(self):
        """
        Send a :attr:`signal.SIGTERM` to the process.
        Please consider using :meth:`stop` instead if you want to
        do all that's possible to finalize the process and wait for it to finish.
        """
        self._init_subprocess()
        self.send_signal(signal.SIGTERM)

    def kill(self):
        """
        Send a :attr:`signal.SIGKILL` to the process.
        Please consider using :meth:`stop` instead if you want to
        do all that's possible to finalize the process and wait for it to finish.
        """
        self._init_subprocess()
        self.send_signal(signal.SIGKILL)

    def send_signal(self, sig):
        """
        Send the specified signal to the process.

        :param sig: Signal to send.
        """
        self._init_subprocess()
        if self.is_sudo_enabled():
            pids = get_children_pids(self.get_pid())
            pids.append(self.get_pid())
            for pid in pids:
                kill_cmd = 'kill -%d %d' % (int(sig), pid)
                with contextlib.suppress(Exception):
                    run(kill_cmd, sudo=True)
        else:
            self._popen.send_signal(sig)

    def poll(self):
        """
        Call the subprocess poll() method, fill results if rc is not None.
        """
        self._init_subprocess()
        rc = self._popen.poll()
        if rc is not None:
            self._fill_results(rc)
        return rc

    def wait(self, timeout=None, sig=signal.SIGTERM):
        """
        Call the subprocess poll() method, fill results if rc is not None.

        :param timeout: Time (seconds) we'll wait until the process is
                        finished. If it's not, we'll try to terminate it
                        and it's children using ``sig`` and get a
                        status. When the process refuses to die
                        within 1s we use SIGKILL and report the status
                        (be it exit_code or zombie)
        :param sig: Signal to send to the process in case it did not end after
                    the specified timeout.
        """
        def nuke_myself():
            self.result.interrupted = ("timeout after %ss"
                                       % (time.time() - self.start_time))
            try:
                kill_process_tree(self.get_pid(), sig, timeout=1)
            except RuntimeError:
                try:
                    kill_process_tree(self.get_pid(), signal.SIGKILL,
                                      timeout=1)
                    log.warning("Process '%s' refused to die in 1s after "
                                "sending %s to, destroyed it successfully "
                                "using SIGKILL.", self.cmd, sig)
                except RuntimeError:
                    log.error("Process '%s' refused to die in 1s after "
                              "sending %s, followed by SIGKILL, probably "
                              "dealing with a zombie process.", self.cmd,
                              sig)

        self._init_subprocess()
        rc = None

        if timeout is None:
            rc = self._popen.wait()
        elif timeout > 0.0:
            timer = threading.Timer(timeout, nuke_myself)
            try:
                timer.start()
                rc = self._popen.wait()
            finally:
                timer.cancel()

        if rc is None:
            stop_time = time.time() + 1
            while time.time() < stop_time:
                rc = self._popen.poll()
                if rc is not None:
                    break
            else:
                nuke_myself()
                rc = self._popen.poll()

        if rc is None:
            # If all this work fails, we're dealing with a zombie process.
            raise AssertionError('Zombie Process %s' % self._popen.pid)
        self._fill_results(rc)
        return rc

    def stop(self, timeout=None):
        """
        Stop background subprocess.

        Call this method to terminate the background subprocess and
        wait for it results.

        :param timeout: Time (seconds) we'll wait until the process is
                        finished. If it's not, we'll try to terminate it
                        and it's children using ``sig`` and get a
                        status. When the process refuses to die
                        within 1s we use SIGKILL and report the status
                        (be it exit_code or zombie)
        """
        self._init_subprocess()
        if self.result.exit_status is None:
            self.terminate()
        return self.wait(timeout)

    def get_pid(self):
        """
        Reports PID of this process
        """
        self._init_subprocess()
        return self._popen.pid

    def get_user_id(self):
        """
        Reports user id of this process
        """
        self._init_subprocess()
        return get_owner_id(self.get_pid())

    def is_sudo_enabled(self):
        """
        Returns whether the subprocess is running with sudo enabled
        """
        self._init_subprocess()
        return self.get_user_id() == 0

    def run(self, timeout=None, sig=signal.SIGTERM):
        """
        Start a process and wait for it to end, returning the result attr.

        If the process was already started using .start(), this will simply
        wait for it to end.

        :param timeout: Time (seconds) we'll wait until the process is
                        finished. If it's not, we'll try to terminate it
                        and it's children using ``sig`` and get a
                        status. When the process refuses to die
                        within 1s we use SIGKILL and report the status
                        (be it exit_code or zombie)
        :type timeout: float
        :param sig: Signal to send to the process in case it did not end after
                    the specified timeout.
        :type sig: int
        :returns: The command result object.
        :rtype: A :class:`CmdResult` instance.
        """
        self._init_subprocess()
        self.wait(timeout, sig)
        return self.result


class WrapSubProcess(SubProcess):

    """
    Wrap subprocess inside an utility program.
    """

    def __init__(self, cmd, verbose=True,
                 allow_output_check=None,
                 shell=False, env=None, wrapper=None, sudo=False,
                 ignore_bg_processes=False, encoding=None):
        if wrapper is None and CURRENT_WRAPPER is not None:
            wrapper = CURRENT_WRAPPER
        self.wrapper = wrapper
        if self.wrapper:
            if not os.path.exists(self.wrapper):
                raise IOError("No such wrapper: '%s'" % self.wrapper)
            cmd = wrapper + ' ' + cmd
        super(WrapSubProcess, self).__init__(cmd, verbose, allow_output_check,
                                             shell, env, sudo,
                                             ignore_bg_processes, encoding)


def should_run_inside_wrapper(cmd):
    """
    Whether the given command should be run inside the wrapper utility.

    :param cmd: the command arguments, from where we extract the binary name
    """
    global CURRENT_WRAPPER  # pylint: disable=W0603
    CURRENT_WRAPPER = None
    args = shlex.split(cmd)
    cmd_binary_name = args[0]

    for script, cmd_expr in WRAP_PROCESS_NAMES_EXPR:
        if fnmatch.fnmatch(cmd_binary_name, cmd_expr):
            CURRENT_WRAPPER = script

    if WRAP_PROCESS is not None and CURRENT_WRAPPER is None:
        CURRENT_WRAPPER = WRAP_PROCESS

    if CURRENT_WRAPPER is None:
        return False
    else:
        return True


def get_sub_process_klass(cmd):
    """
    Which sub process implementation should be used

    Either the regular one, or the GNU Debugger version

    :param cmd: the command arguments, from where we extract the binary name
    """
    if should_run_inside_wrapper(cmd):
        return WrapSubProcess
    else:
        return SubProcess


def run(cmd, timeout=None, verbose=True, ignore_status=False,
        allow_output_check=None, shell=False,
        env=None, sudo=False, ignore_bg_processes=False,
        encoding=None):
    """
    Run a subprocess, returning a CmdResult object.

    :param cmd: Command line to run.
    :type cmd: str
    :param timeout: Time limit in seconds before attempting to kill the
                    running process. This function will take a few seconds
                    longer than 'timeout' to complete if it has to kill the
                    process.
    :type timeout: float
    :param verbose: Whether to log the command run and stdout/stderr.
    :type verbose: bool
    :param ignore_status: Whether to raise an exception when command returns
                          =! 0 (False), or not (True).
    :type ignore_status: bool
    :param allow_output_check: Whether to record the output from this
                               process (from stdout and stderr) in the
                               test's output record files. Valid values:
                               'stdout', for standard output *only*,
                               'stderr' for standard error *only*,
                               'both' for both standard output and error
                               in separate files, 'combined' for
                               standard output and error in a single file,
                               and 'none' to disable all recording. 'all'
                               is also a valid, but deprecated, option that
                               is a synonym of 'both'.  If an explicit value
                               is not given to this parameter, that is, if
                               None is given, it defaults to using the module
                               level configuration, as set by
                               :data:`OUTPUT_CHECK_RECORD_MODE`.  If the
                               module level configuration itself is not set,
                               it defaults to 'none'.
    :type allow_output_check: str
    :param shell: Whether to run the command on a subshell
    :type shell: bool
    :param env: Use extra environment variables
    :type env: dict
    :param sudo: Whether the command requires admin privileges to run,
                 so that sudo will be prepended to the command.
                 The assumption here is that the user running the command
                 has a sudo configuration such that a password won't be
                 prompted. If that's not the case, the command will
                 straight out fail.
    :param encoding: the encoding to use for the text representation
                     of the command result stdout and stderr, by default
                     :data:`avocado.utils.astring.ENCODING`
    :type encoding: str

    :return: An :class:`CmdResult` object.
    :raise: :class:`CmdError`, if ``ignore_status=False``.
    """
    if encoding is None:
        encoding = astring.ENCODING
    klass = get_sub_process_klass(cmd)
    sp = klass(cmd=cmd, verbose=verbose,
               allow_output_check=allow_output_check, shell=shell, env=env,
               sudo=sudo, ignore_bg_processes=ignore_bg_processes,
               encoding=encoding)
    cmd_result = sp.run(timeout=timeout)
    fail_condition = cmd_result.exit_status != 0 or cmd_result.interrupted
    if fail_condition and not ignore_status:
        raise CmdError(cmd, sp.result)
    return cmd_result


def system(cmd, timeout=None, verbose=True, ignore_status=False,
           allow_output_check=None, shell=False,
           env=None, sudo=False, ignore_bg_processes=False,
           encoding=None):
    """
    Run a subprocess, returning its exit code.

    :param cmd: Command line to run.
    :type cmd: str
    :param timeout: Time limit in seconds before attempting to kill the
                    running process. This function will take a few seconds
                    longer than 'timeout' to complete if it has to kill the
                    process.
    :type timeout: float
    :param verbose: Whether to log the command run and stdout/stderr.
    :type verbose: bool
    :param ignore_status: Whether to raise an exception when command returns
                          =! 0 (False), or not (True).
    :type ignore_status: bool
    :param allow_output_check: Whether to record the output from this
                               process (from stdout and stderr) in the
                               test's output record files. Valid values:
                               'stdout', for standard output *only*,
                               'stderr' for standard error *only*,
                               'both' for both standard output and error
                               in separate files, 'combined' for
                               standard output and error in a single file,
                               and 'none' to disable all recording. 'all'
                               is also a valid, but deprecated, option that
                               is a synonym of 'both'.  If an explicit value
                               is not given to this parameter, that is, if
                               None is given, it defaults to using the module
                               level configuration, as set by
                               :data:`OUTPUT_CHECK_RECORD_MODE`.  If the
                               module level configuration itself is not set,
                               it defaults to 'none'.
    :type allow_output_check: str
    :param shell: Whether to run the command on a subshell
    :type shell: bool
    :param env: Use extra environment variables.
    :type env: dict
    :param sudo: Whether the command requires admin privileges to run,
                 so that sudo will be prepended to the command.
                 The assumption here is that the user running the command
                 has a sudo configuration such that a password won't be
                 prompted. If that's not the case, the command will
                 straight out fail.
    :param encoding: the encoding to use for the text representation
                     of the command result stdout and stderr, by default
                     :data:`avocado.utils.astring.ENCODING`
    :type encoding: str

    :return: Exit code.
    :rtype: int
    :raise: :class:`CmdError`, if ``ignore_status=False``.
    """
    cmd_result = run(cmd=cmd, timeout=timeout, verbose=verbose, ignore_status=ignore_status,
                     allow_output_check=allow_output_check, shell=shell, env=env,
                     sudo=sudo, ignore_bg_processes=ignore_bg_processes,
                     encoding=encoding)
    return cmd_result.exit_status


def system_output(cmd, timeout=None, verbose=True, ignore_status=False,
                  allow_output_check=None, shell=False,
                  env=None, sudo=False, ignore_bg_processes=False,
                  strip_trail_nl=True, encoding=None):
    """
    Run a subprocess, returning its output.

    :param cmd: Command line to run.
    :type cmd: str
    :param timeout: Time limit in seconds before attempting to kill the
                    running process. This function will take a few seconds
                    longer than 'timeout' to complete if it has to kill the
                    process.
    :type timeout: float
    :param verbose: Whether to log the command run and stdout/stderr.
    :type verbose: bool
    :param ignore_status: Whether to raise an exception when command returns
                          =! 0 (False), or not (True).
    :param allow_output_check: Whether to record the output from this
                               process (from stdout and stderr) in the
                               test's output record files. Valid values:
                               'stdout', for standard output *only*,
                               'stderr' for standard error *only*,
                               'both' for both standard output and error
                               in separate files, 'combined' for
                               standard output and error in a single file,
                               and 'none' to disable all recording. 'all'
                               is also a valid, but deprecated, option that
                               is a synonym of 'both'.  If an explicit value
                               is not given to this parameter, that is, if
                               None is given, it defaults to using the module
                               level configuration, as set by
                               :data:`OUTPUT_CHECK_RECORD_MODE`.  If the
                               module level configuration itself is not set,
                               it defaults to 'none'.
    :type allow_output_check: str
    :param shell: Whether to run the command on a subshell
    :type shell: bool
    :param env: Use extra environment variables
    :type env: dict
    :param sudo: Whether the command requires admin privileges to run,
                 so that sudo will be prepended to the command.
                 The assumption here is that the user running the command
                 has a sudo configuration such that a password won't be
                 prompted. If that's not the case, the command will
                 straight out fail.
    :type sudo: bool
    :param ignore_bg_processes: Whether to ignore background processes
    :type ignore_bg_processes: bool
    :param strip_trail_nl: Whether to strip the trailing newline
    :type strip_trail_nl: bool
    :param encoding: the encoding to use for the text representation
                     of the command result stdout and stderr, by default
                     :data:`avocado.utils.astring.ENCODING`
    :type encoding: str

    :return: Command output.
    :rtype: bytes
    :raise: :class:`CmdError`, if ``ignore_status=False``.
    """
    cmd_result = run(cmd=cmd, timeout=timeout, verbose=verbose, ignore_status=ignore_status,
                     allow_output_check=allow_output_check, shell=shell, env=env,
                     sudo=sudo, ignore_bg_processes=ignore_bg_processes,
                     encoding=encoding)
    if strip_trail_nl:
        return cmd_result.stdout.rstrip(b'\n\r')
    return cmd_result.stdout


def getoutput(cmd, timeout=None, verbose=False, ignore_status=True,
              allow_output_check='combined', shell=True,
              env=None, sudo=False, ignore_bg_processes=False):
    """
    Because commands module is removed in Python3 and it redirect stderr
    to stdout, we port commands.getoutput to make code compatible
    Return output (stdout or stderr) of executing cmd in a shell.

    :param cmd: Command line to run.
    :type cmd: str
    :param timeout: Time limit in seconds before attempting to kill the
                    running process. This function will take a few seconds
                    longer than 'timeout' to complete if it has to kill the
                    process.
    :type timeout: float
    :param verbose: Whether to log the command run and stdout/stderr.
    :type verbose: bool
    :param ignore_status: Whether to raise an exception when command returns
                          =! 0 (False), or not (True).
    :param allow_output_check: Whether to record the output from this
                               process (from stdout and stderr) in the
                               test's output record files. Valid values:
                               'stdout', for standard output *only*,
                               'stderr' for standard error *only*,
                               'both' for both standard output and error
                               in separate files, 'combined' for
                               standard output and error in a single file,
                               and 'none' to disable all recording. 'all'
                               is also a valid, but deprecated, option that
                               is a synonym of 'both'.  If an explicit value
                               is not given to this parameter, that is, if
                               None is given, it defaults to using the module
                               level configuration, as set by
                               :data:`OUTPUT_CHECK_RECORD_MODE`.  If the
                               module level configuration itself is not set,
                               it defaults to 'none'.
    :type allow_output_check: str
    :param shell: Whether to run the command on a subshell
    :type shell: bool
    :param env: Use extra environment variables
    :type env: dict
    :param sudo: Whether the command requires admin privileges to run,
                 so that sudo will be prepended to the command.
                 The assumption here is that the user running the command
                 has a sudo configuration such that a password won't be
                 prompted. If that's not the case, the command will
                 straight out fail.
    :type sudo: bool
    :param ignore_bg_processes: Whether to ignore background processes
    :type ignore_bg_processes: bool

    :return: Command output(stdout or stderr).
    :rtype: str
    """
    return getstatusoutput(cmd=cmd, timeout=timeout, verbose=verbose,
                           ignore_status=ignore_status,
                           allow_output_check=allow_output_check, shell=shell, env=env,
                           sudo=sudo, ignore_bg_processes=ignore_bg_processes)[1]


def getstatusoutput(cmd, timeout=None, verbose=False, ignore_status=True,
                    allow_output_check='combined', shell=True,
                    env=None, sudo=False, ignore_bg_processes=False):
    """
    Because commands module is removed in Python3 and it redirect stderr
    to stdout, we port commands.getstatusoutput to make code compatible
    Return (status, output) of executing cmd in a shell.

    :param cmd: Command line to run.
    :type cmd: str
    :param timeout: Time limit in seconds before attempting to kill the
                    running process. This function will take a few seconds
                    longer than 'timeout' to complete if it has to kill the
                    process.
    :type timeout: float
    :param verbose: Whether to log the command run and stdout/stderr.
    :type verbose: bool
    :param ignore_status: Whether to raise an exception when command returns
                          =! 0 (False), or not (True).
    :param allow_output_check: Whether to record the output from this
                               process (from stdout and stderr) in the
                               test's output record files. Valid values:
                               'stdout', for standard output *only*,
                               'stderr' for standard error *only*,
                               'both' for both standard output and error
                               in separate files, 'combined' for
                               standard output and error in a single file,
                               and 'none' to disable all recording. 'all'
                               is also a valid, but deprecated, option that
                               is a synonym of 'both'.  If an explicit value
                               is not given to this parameter, that is, if
                               None is given, it defaults to using the module
                               level configuration, as set by
                               :data:`OUTPUT_CHECK_RECORD_MODE`.  If the
                               module level configuration itself is not set,
                               it defaults to 'none'.
    :type allow_output_check: str
    :param shell: Whether to run the command on a subshell
    :type shell: bool
    :param env: Use extra environment variables
    :type env: dict
    :param sudo: Whether the command requires admin privileges to run,
                 so that sudo will be prepended to the command.
                 The assumption here is that the user running the command
                 has a sudo configuration such that a password won't be
                 prompted. If that's not the case, the command will
                 straight out fail.
    :type sudo: bool
    :param ignore_bg_processes: Whether to ignore background processes
    :type ignore_bg_processes: bool

    :return: Exit status and command output(stdout and stderr).
    :rtype: tuple
    """
    cmd_result = run(cmd=cmd, timeout=timeout, verbose=verbose, ignore_status=ignore_status,
                     allow_output_check=allow_output_check, shell=shell, env=env,
                     sudo=sudo, ignore_bg_processes=ignore_bg_processes)
    text = cmd_result.stdout_text
    sts = cmd_result.exit_status
    if text[-1:] == '\n':
        text = text[:-1]
    return (sts, text)


def get_owner_id(pid):
    """
    Get the owner's user id of a process

    :param pid: the process id
    :return: user id of the process owner
    """
    try:
        return os.stat('/proc/%d/' % pid).st_uid
    except OSError:
        return None


def get_command_output_matching(command, pattern):
    """
    Runs a command, and if the pattern is in in the output, returns it.

    :param command: the command to execute
    :type command: str
    :param pattern: pattern to search in the output, in a line by line basis
    :type pattern: str

    :return: list of lines matching the pattern
    :rtype: list of str
    """
    return [line for line in run(command).stdout_text.splitlines()
            if pattern in line]



import logging
import os
import sys
import tempfile
import unittest

import pkg_resources

#: The base directory for the avocado source tree
BASEDIR = os.path.dirname(os.path.abspath(__file__))
BASEDIR = os.path.abspath(os.path.join(BASEDIR, os.path.pardir))

#: The name of the avocado test runner entry point
AVOCADO = os.environ.get("UNITTEST_AVOCADO_CMD",
                         "%s -m avocado" % sys.executable)


def python_module_available(module_name):
    '''
    Checks if a given Python module is available

    :param module_name: the name of the module
    :type module_name: str
    :returns: if the Python module is available in the system
    :rtype: bool
    '''
    try:
        pkg_resources.require(module_name)
        return True
    except pkg_resources.DistributionNotFound:
        return False


def setup_avocado_loggers():
    """
    Setup avocado loggers to contain at least one logger

    This is required for tests that directly utilize avocado.Test classes
    because they require those loggers to be configured. Without this
    it might (py2) result in infinite recursion while attempting to log
    "No handlers could be found for logger ..." message.
    """
    for name in ('', 'avocado.test', 'avocado.app'):
        logger = logging.getLogger(name)
        if not logger.handlers:
            logger.handlers.append(logging.NullHandler())


def temp_dir_prefix(module_name, klass, method):
    """
    Returns a standard name for the temp dir prefix used by the tests
    """
    fmt = 'avocado__%s__%s__%s__'
    return fmt % (module_name, klass.__class__.__name__, method)


def get_temporary_config(module_name, klass, method):
    """
    Creates a temporary bogus config file
    returns base directory, dictionary containing the temporary data dir
    paths and the configuration file contain those same settings
    """
    prefix = temp_dir_prefix(module_name, klass, method)
    base_dir = tempfile.TemporaryDirectory(prefix=prefix)
    test_dir = os.path.join(base_dir.name, 'tests')
    os.mkdir(test_dir)
    data_directory = os.path.join(base_dir.name, 'data')
    os.mkdir(data_directory)
    cache_dir = os.path.join(data_directory, 'cache')
    os.mkdir(cache_dir)
    mapping = {'base_dir': base_dir.name,
               'test_dir': test_dir,
               'data_dir': data_directory,
               'logs_dir': os.path.join(base_dir.name, 'logs'),
               'cache_dir': cache_dir}
    temp_settings = ('[datadir.paths]\n'
                     'base_dir = %(base_dir)s\n'
                     'test_dir = %(test_dir)s\n'
                     'data_dir = %(data_dir)s\n'
                     'cache_dirs = ["%(cache_dir)s"]\n'
                     'logs_dir = %(logs_dir)s\n') % mapping
    config_file = tempfile.NamedTemporaryFile('w', delete=False)
    config_file.write(temp_settings)
    config_file.close()
    return base_dir, mapping, config_file


#: The plugin module names and directories under optional_plugins
PLUGINS = {'varianter_yaml_to_mux': 'avocado-framework-plugin-varianter-yaml-to-mux',
           'runner_remote': 'avocado-framework-plugin-runner-remote',
           'runner_vm': 'avocado-framework-plugin-runner-vm',
           'varianter_cit': 'avocado-framework-plugin-varianter-cit',
           'html': 'avocado-framework-plugin-result-html'}


def test_suite(base_selftests=True, plugin_selftests=None):
    '''
    Returns a test suite with all selftests found

    This includes tests on available optional plugins directories

    :param base_selftests: if the base selftests directory should be included
    :type base_selftests: bool
    :param plugin_selftests: the list optional plugin directories to include
                             or None to include all
    :type plugin_selftests: list or None
    :rtype: unittest.TestSuite
    '''
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()
    selftests_dir = os.path.dirname(os.path.abspath(__file__))
    basedir = os.path.dirname(selftests_dir)
    if base_selftests:
        for section in ('unit', 'functional'):
            start_dir = os.path.join(selftests_dir, section)
            suite.addTests(loader.discover(start_dir=start_dir,
                                           top_level_dir=basedir))
    if plugin_selftests is None:
        plugin_selftests = list(PLUGINS.keys())
    for plugin_dir in plugin_selftests:
        plugin_name = PLUGINS.get(plugin_dir, None)
        if python_module_available(plugin_name):
            path = os.path.join(basedir, 'optional_plugins',
                                plugin_dir, 'tests')
            suite.addTests(loader.discover(start_dir=path, top_level_dir=path))
    return suite


def skipOnLevelsInferiorThan(level):
    return unittest.skipIf(int(os.environ.get("AVOCADO_CHECK_LEVEL", 0)) < level,
                           "Skipping test that take a long time to run, are "
                           "resource intensive or time sensitve")


def skipUnlessPathExists(path):
    return unittest.skipUnless(os.path.exists(path),
                               ('File or directory at path "%s" used in test is'
                                ' not available in the system' % path))


class TestCaseTmpDir(unittest.TestCase):

    def setUp(self):
        prefix = temp_dir_prefix(__name__, self, 'setUp')
        self.tmpdir = tempfile.TemporaryDirectory(prefix=prefix)
        os.chdir(BASEDIR)

    def tearDown(self):
        self.tmpdir.cleanup()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2015-2019
# Author: Cleber Rosa <cleber@redhat.com>

"""
Extension manager with disable/ordering support
"""

from .extension_manager import ExtensionManager
from .settings import settings


class EnabledExtensionManager(ExtensionManager):

    def __init__(self, namespace, invoke_kwds=None):
        super(EnabledExtensionManager, self).__init__(namespace, invoke_kwds)
        namespace = "%s.order" % self.settings_section()
        configured_order = settings.as_dict().get(namespace)
        ordered = []
        if configured_order:
            for name in configured_order:
                for ext in self.extensions:
                    if name == ext.name:
                        ordered.append(ext)
            for ext in self.extensions:
                if ext not in ordered:
                    ordered.append(ext)
            self.extensions = ordered

    def enabled(self, extension):
        """
        Checks configuration for explicit mention of plugin in a disable list

        If configuration section or key doesn't exist, it means no plugin
        is disabled.
        """
        disabled = settings.as_dict().get('plugins.disable')
        return self.fully_qualified_name(extension) not in disabled



# -*- coding: utf-8 -*-
"""This is used as config file to generate Avocado's documentation."""

import errno
import importlib
import os
import sys

from avocado.core import parameters
from avocado.core.varianter import Varianter
from avocado.utils import genio  # pylint: disable=C0413
from avocado.utils import path  # pylint: disable=C0413
from avocado.utils import process  # pylint: disable=C0413

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
ROOT_PATH = os.path.abspath(os.path.join("..", ".."))
sys.path.insert(0, ROOT_PATH)


# Flag that tells if the docs are being built on readthedocs.org
ON_RTD = os.environ.get('READTHEDOCS', None) == 'True'

#
# Auto generate API documentation
#
API_SOURCE_DIR = os.path.join(ROOT_PATH, 'avocado')
BASE_API_OUTPUT_DIR = os.path.join(ROOT_PATH, 'docs', 'source', 'api')
try:
    APIDOC = path.find_command('sphinx-apidoc')
    APIDOC_TEMPLATE = APIDOC + " -o %(output_dir)s %(API_SOURCE_DIR)s %(exclude_dirs)s"
except path.CmdNotFoundError:
    APIDOC = False


def generate_reference():
    avocado = os.path.join(API_SOURCE_DIR, '__main__.py')
    result = process.run("%s %s  config reference" % (sys.executable, avocado))
    reference_path = os.path.join(ROOT_PATH, 'docs', 'source',
                                  'config', 'reference.rst')
    with open(reference_path, 'w') as reference:
        reference.write(result.stdout_text)


def generate_vmimage_distro():
    yaml_path = [os.path.join(ROOT_PATH, 'selftests', 'pre_release', 'tests',
                              'vmimage.py.data', 'variants.yml')]
    reference_dir_path = os.path.join(ROOT_PATH, 'docs', 'source', 'guides',
                                      'writer', 'libs', 'data', 'vmimage')
    reference_path = os.path.join(reference_dir_path, 'supported_images.csv')

    config = {'yaml_to_mux.files': yaml_path, 'yaml_to_mux.inject': []}
    varianter = Varianter()
    varianter.parse(config)

    try:
        os.makedirs(reference_dir_path)
    except FileExistsError:
        pass

    with open(reference_path, 'w') as reference:
        reference.write("Provider, Version, Architecture\n")
        for v in varianter.itertests():
            vmimage_params = parameters.AvocadoParams(v['variant'], ['/run/*'])
            vmimage_name = vmimage_params.get('name')
            vmimage_version = vmimage_params.get('version')
            vmimage_arch = vmimage_params.get('arch', path='*/architectures/*')
            distro_arch_path = '/run/distro/%s/%s/*' % (vmimage_name,
                                                        vmimage_arch)
            vmimage_arch = vmimage_params.get('arch', path=distro_arch_path,
                                              default=vmimage_arch)
            reference.write("%s,%s,%s\n" % (str(vmimage_name),
                                            str(vmimage_version),
                                            str(vmimage_arch)))


generate_reference()
generate_vmimage_distro()

# Documentation sections. Key is the name of the section, followed by:
# Second level module name (after avocado), Module description,
# Output directory, List of directory to exclude from API  generation,
# list of (duplicated) generated reST files to remove (and avoid warnings)
# References tag
API_SECTIONS = {"Test APIs": (None,
                              genio.read_file("api/headers/test"),
                              "test",
                              ("core", "utils", "plugins"),
                              ("modules.rst", ),
                              ".. _tests-api-reference:\n"),
                "Utilities APIs": ("utils",
                                   genio.read_file("api/headers/utils"),
                                   "utils",
                                   ("core", "plugins"),
                                   ("avocado.rst", "modules.rst"),
                                   ""),
                "Internal (Core) APIs": ("core",
                                         genio.read_file("api/headers/core"),
                                         "core",
                                         ("utils", "plugins"),
                                         ("avocado.rst", "modules.rst"),
                                         ""),
                "Extension (plugin) APIs": ("plugins",
                                            genio.read_file("api/headers/plugins"),
                                            "plugins",
                                            ("core", "utils"),
                                            ("avocado.rst", "modules.rst"),
                                            "")}

# clean up all previous rst files. RTD is known to keep them from previous runs
process.run("find %s -name '*.rst' -delete" % BASE_API_OUTPUT_DIR)

for (section, params) in API_SECTIONS.items():
    output_dir = os.path.join(BASE_API_OUTPUT_DIR, params[2])
    exclude_dirs = [os.path.join(API_SOURCE_DIR, d)
                    for d in params[3]]
    exclude_dirs = " ".join(exclude_dirs)
    files_to_remove = [os.path.join(BASE_API_OUTPUT_DIR, output_dir, d)
                       for d in params[4]]

    # generate all rst files
    if APIDOC:
        cmd = APIDOC_TEMPLATE % locals()
        process.run(cmd)
        # remove unnecessary ones
        for f in files_to_remove:
            os.unlink(f)
    # rewrite first lines of main rst file for this section
    second_level_module_name = params[0]
    if second_level_module_name is None:
        main_rst = os.path.join(output_dir,
                                "avocado.rst")
    else:
        main_rst = os.path.join(output_dir,
                                "avocado.%s.rst" % second_level_module_name)
    if not APIDOC:
        main_rst_content = []
        try:
            os.makedirs(os.path.dirname(main_rst))
        except OSError as details:
            if details.errno != errno.EEXIST:
                raise
    else:
        with open(main_rst) as main_rst_file:
            main_rst_content = main_rst_file.readlines()

    new_main_rst_content = [params[5], section, "=" * len(section), "",
                            params[1], ""]
    with open(main_rst, "w") as new_main_rst:
        new_main_rst.write("\n".join(new_main_rst_content))
        new_main_rst.write("".join(main_rst_content[2:]))

# Generate optional-plugins
OPTIONAL_PLUGINS_PATH = os.path.join(ROOT_PATH, "optional_plugins")
API_OPTIONAL_PLUGINS_PATH = os.path.join(BASE_API_OUTPUT_DIR,
                                         "optional-plugins")
if not os.path.exists(API_OPTIONAL_PLUGINS_PATH):
    os.makedirs(API_OPTIONAL_PLUGINS_PATH)
with open(os.path.join(API_OPTIONAL_PLUGINS_PATH, "index.rst"),
          'w') as optional_plugins_toc:
    optional_plugins_toc.write(""".. index file for optional plugins API

====================
Optional Plugins API
====================

The following pages document the private APIs of optional Avocado plugins.

.. toctree::
   :maxdepth: 1

    """)
    for path in next(os.walk(OPTIONAL_PLUGINS_PATH))[1]:
        name = "avocado_%s" % os.path.basename(path)
        try:
            importlib.import_module(name)
        except ImportError:
            continue

        path = os.path.join(OPTIONAL_PLUGINS_PATH, path, name)
        if not os.path.exists(path):
            continue
        output_dir = os.path.join(API_OPTIONAL_PLUGINS_PATH, name)
        params = {"API_SOURCE_DIR": path, "output_dir": output_dir,
                  "exclude_dirs": ""}
        process.run(APIDOC_TEMPLATE % params)
        # Remove the unnecessary generated files
        os.unlink(os.path.join(output_dir, "modules.rst"))
        optional_plugins_toc.write("\n   %s" % os.path.join(name, name))

extensions = ['sphinx.ext.autodoc',  # pylint: disable=C0103
              'sphinx.ext.intersphinx',
              'sphinx.ext.todo',
              'sphinx.ext.coverage']

master_doc = 'index'  # pylint: disable=C0103
project = u'Avocado'  # pylint: disable=C0103
copyright = u'2014-2019, Red Hat'   # pylint: disable=W0622,C0103

VERSION_FILE = os.path.join(ROOT_PATH, 'VERSION')
VERSION = genio.read_file(VERSION_FILE).strip()
version = VERSION  # pylint: disable=C0103
release = VERSION  # pylint: disable=C0103

if not ON_RTD:  # only import and set the theme if we're building docs locally
    try:
        import sphinx_rtd_theme
        html_theme = 'sphinx_rtd_theme'  # pylint: disable=C0103
        html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]  # pylint: disable=C0103
    except ImportError:
        html_theme = 'default'  # pylint: disable=C0103

htmlhelp_basename = 'avocadodoc'  # pylint: disable=C0103

latex_documents = [  # pylint: disable=C0103
    ('index', 'avocado.tex', u'avocado Documentation',
     u'Avocado Development Team', 'manual'),
]

man_pages = [  # pylint: disable=C0103
    ('index', 'avocado', u'avocado Documentation',
     [u'Avocado Development Team'], 1)
]

texinfo_documents = [  # pylint: disable=C0103
    ('index', 'avocado', u'avocado Documentation',
     u'Avocado Development Team', 'avocado', 'One line description of project.',
     'Miscellaneous'),
]

intersphinx_mapping = {'http://docs.python.org/': None}  # pylint: disable=C0103

autoclass_content = 'both'  # pylint: disable=C0103



import os

from avocado import Test, fail_on
from avocado.utils import archive, build, process


class Hello(Test):

    def setUp(self):
        mirrors = ['https://mirrors.peers.community/mirrors/gnu/hello/',
                   'https://mirrors.kernel.org/gnu/hello/',
                   'http://gnu.c3sl.ufpr.br/ftp/',
                   'ftp://ftp.funet.fi/pub/gnu/prep/hello/']
        hello = 'hello-2.9.tar.gz'
        hello_locations = ["%s%s" % (loc, hello) for loc in mirrors]
        hello_sig = 'hello-2.9.tar.gz.sig'
        hello_sig_locations = ["%s%s" % (loc, hello_sig) for loc in mirrors]
        self.hello = self.fetch_asset(
            name=hello,
            locations=hello_locations,
            asset_hash='cb0470b0e8f4f7768338f5c5cfe1688c90fbbc74')
        self.hello_sig = self.fetch_asset(
            name=hello_sig,
            asset_hash='f3b9fae20c35740004ae7b8de1301836dab4ac30',
            locations=hello_sig_locations)

    @fail_on(process.CmdError)
    def test_gpg_signature(self):
        keyring = os.path.join(self.workdir, "tempring.gpg")
        gpg_cmd = "gpg --no-default-keyring --keyring %s" % keyring
        signer_pubkey = self.get_data("gnu_hello_signer.gpg")
        import_cmd = "%s --import %s" % (gpg_cmd, signer_pubkey)
        # gpg will not return 0 when creating a new keyring and
        # importing the public key
        process.run(import_cmd, ignore_status=True)
        verify_cmd = "%s --verify %s %s" % (gpg_cmd,
                                            self.hello_sig,
                                            self.hello)
        process.run(verify_cmd)

    @fail_on(process.CmdError)
    def test_build_run(self):
        hello_src = os.path.join(self.workdir, 'hello-2.9')
        archive.uncompress(self.hello, self.workdir)
        build.configure(hello_src)
        build.make(hello_src)
        process.run(os.path.join(hello_src, 'src', 'hello'))



import os
import sys

from avocado import Test, skip


class SkipSetup(Test):

    @skip("from setUp()")
    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class SkipTest(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    @skip("from test()")
    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class SkipTeardown(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    @skip("from tearDown()")
    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class CancelSetup(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.cancel()
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class CancelTest(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.cancel()
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class CancelTeardown(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.cancel()
        self.log.info('teardown post')


class FailSetup(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.fail()
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class FailTest(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.fail()
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class FailTeardown(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.fail()
        self.log.info('teardown post')


class WarnSetup(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.warn('')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class WarnTest(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.warn('')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class WarnTeardown(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.warn('')
        self.log.info('teardown post')


class ExitSetup(Test):

    def setUp(self):
        self.log.info('setup pre')
        sys.exit(-1)
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class ExitTest(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        sys.exit(-1)
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class ExitTeardown(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        sys.exit(-1)
        self.log.info('teardown post')


class ExceptionSetup(Test):

    def setUp(self):
        self.log.info('setup pre')
        raise ValueError
        # pylint: disable=W0101
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class ExceptionTest(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        raise ValueError
        # pylint: disable=W0101
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')


class ExceptionTeardown(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        raise ValueError
        # pylint: disable=W0101
        self.log.info('teardown post')


class KillTest(Test):

    def setUp(self):
        self.log.info('setup pre')
        self.log.info('setup post')

    def test(self):
        self.log.info('test pre')
        os.kill(os.getpid(), 9)
        self.log.info('test post')

    def tearDown(self):
        self.log.info('teardown pre')
        self.log.info('teardown post')



import unittest

from avocado.core import plugin_interfaces


class Plugin(unittest.TestCase):

    def test_instantiate_settings(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.Settings()

    def test_instantiate_cli(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.CLI()

    def test_instantiate_cli_cmd(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.CLICmd()

    def test_instantiate_job_pre(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.JobPre()

    def test_instantiate_job_post(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.JobPost()

    def test_instantiate_result(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.Result()

    def test_instantiate_job_pre_tests(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.JobPreTests()

    def test_instantiate_job_post_tests(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.JobPostTests()

    def test_instantiate_result_events(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.ResultEvents()

    def test_instantiate_varianter(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.Varianter()

    def test_instantiate_init(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.Init()

    def test_instantiate_spawner(self):
        with self.assertRaises(TypeError):
            # pylint: disable=E0110
            plugin_interfaces.Spawner()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# This code was inspired in the autotest project,
#
# client/base_utils.py
#
# Copyright: 2018 IBM
# Authors : Praveen K Pandey <praveen@linux.vnet.ibm.com>
#           Narasimhan V <sim@linux.vnet.ibm.com>


"""
Disk utilities
"""


import json
import os
import re

from . import process


class DiskError(Exception):
    """
    Generic DiskError
    """


def freespace(path):
    fs_stats = os.statvfs(path)
    return fs_stats.f_bsize * fs_stats.f_bavail


def get_disk_blocksize(path):
    """Return the disk block size, in bytes"""
    fs_stats = os.statvfs(path)
    return fs_stats.f_bsize


def create_loop_device(size, blocksize=4096, directory='./'):
    """
    Creates a loop device of size and blocksize specified.

    :param size: Size of loop device, in bytes
    :type size: int
    :param blocksize: block size of loop device, in bytes. Defaults to 4096
    :type blocksize: int
    :param directory: Directory where the backing file will be created.
                      Defaults to current directory.
    :type directory: str

    :return: loop device name
    :rtype: str
    """
    cmd = "losetup --find"
    loop = process.run(cmd, ignore_status=True,
                       sudo=True).stdout_text.strip('\n')

    loop_file = os.path.join(directory,
                             "tmp_%s.img" % loop.split('/')[-1])
    cmd = "dd if=/dev/zero of={} bs={} count={}".format(loop_file,
                                                        blocksize,
                                                        int(size / blocksize))
    if process.system(cmd, ignore_status=True, sudo=True) != 0:
        raise DiskError("Unable to create backing file for loop device")

    cmd = "losetup %s %s -P" % (loop, loop_file)
    if process.system(cmd, ignore_status=True, sudo=True) != 0:
        raise DiskError("Unable to create the loop device")
    return loop


def delete_loop_device(device):
    """
    Deletes the specified loop device.

    :param device: device to be deleted
    :type device: str

    :return: True if deleted.
    :rtype: bool
    """
    cmd = "losetup -aJl"
    loop_dic = json.loads(process.run(cmd, ignore_status=True,
                                      sudo=True).stdout_text)
    loop_file = ''
    for loop_dev in loop_dic['loopdevices']:
        if device == loop_dev['name']:
            loop_file = loop_dev['back-file']
    if not loop_file:
        raise DiskError("Unable to find backing file for loop device")
    cmd = "losetup -d %s" % device
    if process.system(cmd, ignore_status=True, sudo=True) != 0:
        raise DiskError("Unable to delete the loop device")
    os.remove(loop_file)
    return True


def get_disks():
    """
    Returns the physical "hard drives" available on this system

    This is a simple wrapper around `lsblk` and will return all the
    top level physical (non-virtual) devices return by it.

    TODO: this is currently Linux specific.  Support for other
    platforms is desirable and may be implemented in the future.

    :returns: a list of paths to the physical disks on the system
    :rtype: list of str
    """
    json_result = process.run('lsblk --json --paths --inverse')
    json_data = json.loads(json_result.stdout_text)
    return [str(disk['name']) for disk in json_data['blockdevices']]


def get_available_filesystems():
    """
    Return a list of all available filesystem types

    :returns: a list of filesystem types
    :rtype: list of str
    """
    filesystems = set()
    with open('/proc/filesystems') as proc_fs:
        for proc_fs_line in proc_fs.readlines():
            filesystems.add(re.sub(r'(nodev)?\s*', '', proc_fs_line))
    return list(filesystems)


def get_filesystem_type(mount_point='/'):
    """
    Returns the type of the filesystem of mount point informed.
    The default mount point considered when none is informed
    is the root "/" mount point.

    :param str mount_point: mount point to asses the filesystem type.
                            Default "/"
    :returns: filesystem type
    :rtype: str
    """
    with open('/proc/mounts') as mounts:
        for mount_line in mounts.readlines():
            _, fs_file, fs_vfstype, _, _, _ = mount_line.split()
            if fs_file == mount_point:
                return fs_vfstype


def is_root_device(device):
    """
    check for root disk

    :param device: device to check
    :returns: True or False, True if given device is root disk
              otherwise will return False.
    """
    cmd = "lsblk --j -o MOUNTPOINT,PKNAME"
    output = process.run(cmd)
    result = json.loads(output.stdout_text)
    for item in result['blockdevices']:
        if item['mountpoint'] == "/" and device == str(item['pkname']):
            return True
    return False



from avocado import Test


class CancelTest(Test):

    """
    Example test that cancels the current test from inside the test.
    """

    def test(self):
        self.cancel("This should end with CANCEL.")



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; specifically version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# This code was inspired in the autotest project,
# client/shared/utils.py
# Authors: Martin J Bligh <mbligh@google.com>, Andy Whitcroft <apw@shadowen.org>


"""
Methods to download URLs and regular files.
"""

import logging
import os
import shutil
import socket
from urllib.request import urlopen

from . import aurl, crypto, output

log = logging.getLogger('avocado.test')


def url_open(url, data=None, timeout=5):
    """
    Wrapper to :func:`urllib2.urlopen` with timeout addition.

    :param url: URL to open.
    :param data: (optional) data to post.
    :param timeout: (optional) default timeout in seconds.
    :return: file-like object.
    :raises: `URLError`.
    """
    # Save old timeout
    old_timeout = socket.getdefaulttimeout()
    socket.setdefaulttimeout(timeout)
    try:
        result = urlopen(url, data=data)
        msg = ('Retrieved URL "%s": content-length %s, date: "%s", '
               'last-modified: "%s"')
        logging.debug(msg, url,
                      result.headers.get('Content-Length', 'UNKNOWN'),
                      result.headers.get('Date', 'UNKNOWN'),
                      result.headers.get('Last-Modified', 'UNKNOWN'))
        return result
    finally:
        socket.setdefaulttimeout(old_timeout)


def url_download(url, filename, data=None, timeout=300):
    """
    Retrieve a file from given url.

    :param url: source URL.
    :param filename: destination path.
    :param data: (optional) data to post.
    :param timeout: (optional) default timeout in seconds.
    :return: `None`.
    """
    log.info('Fetching %s -> %s', url, filename)

    src_file = url_open(url, data=data, timeout=timeout)
    try:
        with open(filename, 'wb') as dest_file:
            shutil.copyfileobj(src_file, dest_file)
    finally:
        src_file.close()


def url_download_interactive(url, output_file, title='', chunk_size=102400):
    """
    Interactively downloads a given file url to a given output file.

    :type url: string
    :param url: URL for the file to be download
    :type output_file: string
    :param output_file: file name or absolute path on which to save the file to
    :type title: string
    :param title: optional title to go along the progress bar
    :type chunk_size: integer
    :param chunk_size: amount of data to read at a time
    """
    output_dir = os.path.dirname(output_file)
    with open(output_file, 'w+b') as open_output_file:
        input_file = urlopen(url)

        try:
            file_size = int(input_file.headers['Content-Length'])
        except KeyError:
            raise ValueError('Could not find file size in HTTP headers')

        logging.info('Downloading %s, %s to %s', os.path.basename(url),
                     output.display_data_size(file_size), output_dir)

        progress_bar = output.ProgressBar(maximum=file_size, title=title)

        # Download the file, while interactively updating the progress
        progress_bar.draw()
        while True:
            data = input_file.read(chunk_size)
            if data:
                progress_bar.append_amount(len(data))
                open_output_file.write(data)
            else:
                progress_bar.update_amount(file_size)
                break


def _get_file(src, dst, permissions=None):
    if src == dst:
        return

    if aurl.is_url(src):
        url_download(src, dst)
    else:
        shutil.copyfile(src, dst)

    if permissions:
        os.chmod(dst, permissions)
    return dst


def get_file(src, dst, permissions=None, hash_expected=None,
             hash_algorithm="md5", download_retries=1):
    """
    Gets a file from a source location, optionally using caching.

    If no hash_expected is provided, simply download the file. Else,
    keep trying to download the file until download_failures exceeds
    download_retries or the hashes match.

    If the hashes match, return dst. If download_failures exceeds
    download_retries, raise an EnvironmentError.

    :param src: source path or URL. May be local or a remote file.
    :param dst: destination path.
    :param permissions: (optional) set access permissions.
    :param hash_expected: Hash string that we expect the file downloaded to
            have.
    :param hash_algorithm: Algorithm used to calculate the hash string
            (md5, sha1).
    :param download_retries: Number of times we are going to retry a failed
            download.
    :raise: EnvironmentError.
    :return: destination path.
    """
    def _verify_hash(filename):
        if os.path.isfile(filename):
            return crypto.hash_file(filename, algorithm=hash_algorithm)
        return None

    if hash_expected is None:
        return _get_file(src, dst, permissions)

    download_failures = 0
    hash_file = _verify_hash(dst)

    while not hash_file == hash_expected:
        hash_file = _verify_hash(_get_file(src, dst, permissions))
        if hash_file != hash_expected:
            log.error("It seems that dst %s is corrupted", dst)
            download_failures += 1
        if download_failures > download_retries:
            raise EnvironmentError("Failed to retrieve %s. "
                                   "Possible reasons - Network connectivity "
                                   "problems or incorrect hash_expected "
                                   "provided -> '%s'" %
                                   (src, hash_expected))
        else:
            log.error("Retrying download of src %s", src)

    return dst



import unittest.mock

from avocado.utils import pci


class UtilsPciTest(unittest.TestCase):

    def test_get_slot_from_sysfs(self):
        pcid = '0002:01:00.1'
        file_values = ['S0001', 'S0001[', 'Slot2', 'SLOT1', 'Backplane USB', 'U78CB.001.WZS07CU-P1-C9-T1', 'PLX Slot1', 'Onboard USB', 'U78D5.001.CSS130E-P1-P2-P2-C1-T1']
        expected_values = ['S0001', 'S0001', 'Slot2', 'SLOT1', 'Backplane USB', 'U78CB.001.WZS07CU-P1-C9', 'PLX Slot1', 'Onboard USB', 'U78D5.001.CSS130E-P1-P2-P2-C1']
        for value, exp in zip(file_values, expected_values):
            with unittest.mock.patch('os.path.isfile', return_value=True):
                with unittest.mock.patch('avocado.utils.genio.read_file',
                                         return_value=value):
                    self.assertEqual(pci.get_slot_from_sysfs(pcid), exp)

    def test_get_slot_from_sysfs_negative(self):
        with unittest.mock.patch('os.path.isfile', return_value=True):
            with unittest.mock.patch('avocado.utils.genio.read_file',
                                     return_value='.....bad-value.....'):
                self.assertRaises(ValueError, pci.get_slot_from_sysfs,
                                  '0002:01:00.1')


if __name__ == '__main__':
    unittest.main()



import tempfile

from avocado.core.job import Job
from avocado.utils import script
from avocado.utils.network.ports import find_free_port

from .. import TestCaseTmpDir, skipUnlessPathExists, temp_dir_prefix

SCRIPT_CONTENT = """#!/bin/bash
/bin/sleep 30
"""


class TaskTimeOutTest(TestCaseTmpDir):

    def setUp(self):
        super(TaskTimeOutTest, self).setUp()
        self.script = script.TemporaryScript(
            'sleep.sh',
            SCRIPT_CONTENT,
            'avocado_timeout_functional')
        self.script.save()

        prefix = temp_dir_prefix(__name__, self, 'setUp')
        self.tmpdir = tempfile.TemporaryDirectory(prefix=prefix)

    @skipUnlessPathExists('/bin/sleep')
    def test_sleep_longer_timeout(self):
        status_server = '127.0.0.1:%u' % find_free_port()
        config = {'run.references': [self.script.path],
                  'nrunner.status_server_listen': status_server,
                  'nrunner.status_server_uri': status_server,
                  'run.results_dir': self.tmpdir.name,
                  'run.keep_tmp': True,
                  'task.timeout.running': 2,
                  'run.test_runner': 'nrunner'}

        with Job.from_config(job_config=config) as job:
            job.run()

        self.assertEqual(1, job.result.skipped)
        self.assertEqual(0, job.result.passed)

    def tearDown(self):
        super(TaskTimeOutTest, self).tearDown()
        self.script.remove()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2019
# Authors: Cleber Rosa <crosa@redhat.com>

"""
Test resolver for builtin test types
"""

import os
import re

from avocado.core.nrunner import Runnable
from avocado.core.plugin_interfaces import Resolver
from avocado.core.references import reference_split
from avocado.core.resolver import (ReferenceResolution,
                                   ReferenceResolutionResult, check_file)
from avocado.core.safeloader import find_avocado_tests, find_python_unittests
from avocado.core.settings import settings


class ExecTestResolver(Resolver):

    name = 'exec-test'
    description = 'Test resolver for executable files to be handled as tests'

    @staticmethod
    def resolve(reference):

        criteria_check = check_file(reference, reference, suffix=None,
                                    type_name='executable file',
                                    access_check=os.R_OK | os.X_OK,
                                    access_name='executable')
        if criteria_check is not True:
            return criteria_check

        return ReferenceResolution(reference,
                                   ReferenceResolutionResult.SUCCESS,
                                   [Runnable('exec-test', reference,
                                             config=settings.as_dict(r'^runner\.'))])


def python_resolver(name, reference, find_tests):
    module_path, tests_filter = reference_split(reference)
    if tests_filter is not None:
        tests_filter = re.compile(tests_filter)

    criteria_check = check_file(module_path, reference)
    if criteria_check is not True:
        return criteria_check

    # disabled tests not needed here
    class_methods_info, _ = find_tests(module_path)
    runnables = []
    for klass, methods_tags_reqs in class_methods_info.items():
        for (method, tags, reqs) in methods_tags_reqs:
            klass_method = "%s.%s" % (klass, method)
            if tests_filter is not None and not tests_filter.search(klass_method):
                continue
            uri = "%s:%s" % (module_path, klass_method)
            runnables.append(Runnable(name,
                                      uri=uri,
                                      tags=tags,
                                      requirements=reqs,
                                      config=settings.as_dict(r'^runner\.')))
    if runnables:
        return ReferenceResolution(reference,
                                   ReferenceResolutionResult.SUCCESS,
                                   runnables)

    return ReferenceResolution(reference,
                               ReferenceResolutionResult.NOTFOUND)


class PythonUnittestResolver(Resolver):

    name = 'python-unittest'
    description = 'Test resolver for Python Unittests'

    @staticmethod
    def _find_compat(module_path):
        """Used as compatibility for the :func:`python_resolver()` interface."""
        return find_python_unittests(module_path), None

    @staticmethod
    def resolve(reference):
        return python_resolver(PythonUnittestResolver.name,
                               reference,
                               PythonUnittestResolver._find_compat)


class AvocadoInstrumentedResolver(Resolver):

    name = 'avocado-instrumented'
    description = 'Test resolver for Avocado Instrumented tests'

    @staticmethod
    def resolve(reference):
        return python_resolver(AvocadoInstrumentedResolver.name,
                               reference,
                               find_avocado_tests)


class TapResolver(Resolver):

    name = 'tap'
    description = 'Test resolver for executable files to be handled as tests'

    @staticmethod
    def resolve(reference):

        criteria_check = check_file(reference, reference, suffix=None,
                                    type_name='executable file',
                                    access_check=os.R_OK | os.X_OK,
                                    access_name='executable')
        if criteria_check is not True:
            return criteria_check

        return ReferenceResolution(reference,
                                   ReferenceResolutionResult.SUCCESS,
                                   [Runnable('tap', reference,
                                             config=settings.as_dict(r'^runner\.'))])



# Having 2 imports forces both paths
import avocado


# Should not be discovered as "Test" import did not happened yet
class DontCrash0(Test):
    def test(self):
        pass


from avocado import Test


# on "import avocado" this requires some skipping
class DontCrash1:
    pass


# This one should be discovered no matter how other
# classes break
class DiscoverMe(avocado.Test):
    def test(self):
        pass


# The same as "DontCrash1" only this one should be discovered
class DiscoverMe2(avocado.Test, main):  # pylint: disable=E0240,E0602
    def test(self):
        pass


# The same as "DontCrash1" only this one should be discovered
class DiscoverMe3(Test, main):  # pylint: disable=E0240,E0602
    def test(self):
        pass


class DontCrash2p:
    class Bar(avocado.Test):
        def test(self):
            pass


# Only top-level-namespace classes are allowed for
# in-module-class definitions
class DontCrash2(DontCrash2p.Bar):
    """:avocado: recursive"""


# Class DiscoverMe4p is defined after this one
class DiscoverMe4(DiscoverMe4p):    # pylint: disable=E0601
    """:avocado: recursive"""


class DiscoverMe4p:
    def test(self):
        pass


dont_crash3_on_broken_syntax    # pylint: disable=E0602,W0104



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2015-2019
# Author: Cleber Rosa <cleber@redhat.com>

"""
Base extension manager

This is a mix of stevedore-like APIs and behavior, with Avocado's
own look and feel.
"""

import copy
import logging
import sys

import pkg_resources

from ..utils import stacktrace

# This is also defined in avocado.core.output, but this avoids a
# circular import
LOG_UI = logging.getLogger("avocado.app")


class Extension:
    """
    This is a verbatim copy from the stevedore.extension class with the
    same name
    """

    def __init__(self, name, entry_point, plugin, obj):
        self.name = name
        self.entry_point = entry_point
        self.plugin = plugin
        self.obj = obj


class ExtensionManager:

    #: Default namespace prefix for Avocado extensions
    NAMESPACE_PREFIX = 'avocado.plugins.'

    def __init__(self, namespace, invoke_kwds=None):
        self.namespace = namespace
        self.extensions = []
        self.load_failures = []
        if invoke_kwds is None:
            invoke_kwds = {}

        # load plugins
        for ep in pkg_resources.iter_entry_points(self.namespace):
            try:
                plugin = ep.load()
                obj = plugin(**invoke_kwds)
            except ImportError as exception:
                self.load_failures.append((ep, exception))
            else:
                ext = Extension(ep.name, ep, plugin, obj)
                if self.enabled(ext):  # lgtm [py/init-calls-subclass]
                    self.extensions.append(ext)
        self.extensions.sort(key=lambda x: x.name)

    def enabled(self, extension):  # pylint: disable=W0613
        """
        Checks if a plugin is enabled

        Sub classes can change this implementation to determine their own
        criteria.
        """
        return True

    def plugin_type(self):
        """
        Subset of entry points namespace for this dispatcher

        Given an entry point `avocado.plugins.foo`, plugin type is `foo`.  If
        entry point does not conform to the Avocado standard prefix, it's
        returned unchanged.
        """
        if self.namespace.startswith(self.NAMESPACE_PREFIX):
            return self.namespace[len(self.NAMESPACE_PREFIX):]
        else:
            return self.namespace

    def fully_qualified_name(self, extension):
        """
        Returns the Avocado fully qualified plugin name

        :param extension: an Extension instance
        :type extension: :class:`Extension`
        """
        return "%s.%s" % (self.plugin_type(), extension.entry_point.name)

    def settings_section(self):
        """
        Returns the config section name for the plugin type handled by itself
        """
        return "plugins.%s" % self.plugin_type()

    def names(self):
        """
        Returns the names of the discovered extensions

        This differs from :func:`stevedore.extension.ExtensionManager.names`
        in that it returns names in a predictable order, by using standard
        :func:`sorted`.
        """
        return sorted(super(ExtensionManager, self).names())

    def map_method_with_return(self, method_name, *args, **kwargs):
        """
        The same as `map_method` but additionally reports the list of returned
        values and optionally deepcopies the passed arguments

        :param method_name: Name of the method to be called on each ext
        :param args: Arguments to be passed to all called functions
        :param kwargs: Key-word arguments to be passed to all called functions
                        if `"deepcopy" == True` is present in kwargs the
                        args and kwargs are deepcopied before passing it
                        to each called function.
        """
        deepcopy = kwargs.pop("deepcopy", False)
        ret = []
        for ext in self.extensions:
            try:
                if hasattr(ext.obj, method_name):
                    method = getattr(ext.obj, method_name)
                    if deepcopy:
                        copied_args = [copy.deepcopy(arg) for arg in args]
                        copied_kwargs = copy.deepcopy(kwargs)
                        ret.append(method(*copied_args, **copied_kwargs))
                    else:
                        ret.append(method(*args, **kwargs))
            except SystemExit:
                raise
            except KeyboardInterrupt:
                raise
            except:     # catch any exception pylint: disable=W0702
                stacktrace.log_exc_info(sys.exc_info(),
                                        logger='avocado.app.debug')
                LOG_UI.error('Error running method "%s" of plugin "%s": %s',
                             method_name, ext.name, sys.exc_info()[1])
        return ret

    def map_method(self, method_name, *args):
        """
        Maps method_name on each extension in case the extension has the attr

        :param method_name: Name of the method to be called on each ext
        :param args: Arguments to be passed to all called functions
        """
        for ext in self.extensions:
            try:
                if hasattr(ext.obj, method_name):
                    method = getattr(ext.obj, method_name)
                    method(*args)
            except SystemExit:
                raise
            except KeyboardInterrupt:
                raise
            except:     # catch any exception pylint: disable=W0702
                stacktrace.log_exc_info(sys.exc_info(),
                                        logger='avocado.app.debug')
                LOG_UI.error('Error running method "%s" of plugin "%s": %s',
                             method_name, ext.name, sys.exc_info()[1])

    def __getitem__(self, name):
        for ext in self.extensions:
            if ext.name == name:
                return ext
        raise KeyError

    def __iter__(self):
        return iter(self.extensions)



from avocado.core.output import LOG_UI
from avocado.core.plugin_interfaces import CLICmd
from avocado.core.settings import settings


class HelloWorld(CLICmd):

    name = 'hello'
    description = "The classical Hello World plugin example!"

    def configure(self, parser):
        settings.register_option(section='hello',
                                 key='message',
                                 key_type=str,
                                 default=self.description,
                                 help_msg="Configure the message to display")

    def run(self, config):
        msg = config.get('hello.message')
        LOG_UI.info(msg)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2013-2014
# Author: Ruda Moura <rmoura@redhat.com>

"""
Base Test Runner Plugins.
"""

import argparse
import sys

from avocado.core import exit_codes, job, loader, parser_common_args
from avocado.core.output import LOG_UI
from avocado.core.plugin_interfaces import CLICmd, Init
from avocado.core.settings import settings
from avocado.core.suite import TestSuite, TestSuiteError
from avocado.utils import process


class RunInit(Init):

    name = 'run'
    description = 'Initializes the run options'

    def initialize(self):
        help_msg = ('Defines the order of iterating through test suite '
                    'and test variants')
        settings.register_option(section='run',
                                 key='execution_order',
                                 choices=('tests-per-variant',
                                          'variants-per-test'),
                                 default='variants-per-test',
                                 help_msg=help_msg)


class Run(CLICmd):

    """
    Implements the avocado 'run' subcommand
    """

    name = 'run'
    description = ("Runs one or more tests (native test, test alias, binary "
                   "or script)")

    @staticmethod
    def _test_parameter(string):
        param_name_value = string.split('=', 1)
        if len(param_name_value) < 2:
            msg = ('Invalid --test-parameter option: "%s". Valid option must '
                   'be a "NAME=VALUE" like expression' % string)
            raise argparse.ArgumentTypeError(msg)
        return param_name_value

    def configure(self, parser):
        """
        Add the subparser for the run action.

        :param parser: Main test runner parser.
        """
        parser = super(Run, self).configure(parser)

        help_msg = 'List of test references (aliases or paths)'
        settings.register_option(section='run',
                                 key='references',
                                 key_type=list,
                                 default=[],
                                 nargs='*',
                                 metavar='TEST_REFERENCE',
                                 parser=parser,
                                 help_msg=help_msg,
                                 positional_arg=True)

        help_msg = ('Parameter name and value to pass to all tests. This is '
                    'only applicable when not using a varianter plugin. '
                    'This option format must be given in the NAME=VALUE '
                    'format, and may be given any number of times, or per '
                    'parameter.')
        settings.register_option(section='run',
                                 key='test_parameters',
                                 action='append',
                                 default=[],
                                 key_type=self._test_parameter,
                                 metavar="NAME_VALUE",
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--test-parameter',
                                 short_arg='-p')

        help_msg = ('Selects the runner implementation from one of the '
                    'installed and active implementations.  You can run '
                    '"avocado plugins" and find the list of valid runners '
                    'under the "Plugins that run test suites on a job '
                    '(runners) section.  Defaults to "runner", which is '
                    'the conventional and traditional runner.')
        settings.register_option(section='run',
                                 key='test_runner',
                                 default='runner',
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--test-runner')

        help_msg = ('Instead of running the test only list them and log '
                    'their params.')
        settings.register_option(section='run.dry_run',
                                 key='enabled',
                                 default=False,
                                 key_type=bool,
                                 help_msg=help_msg,
                                 parser=parser,
                                 short_arg='-d',
                                 long_arg='--dry-run')

        help_msg = ('Do not automatically clean up temporary directories '
                    'used by dry-run')
        settings.register_option(section='run.dry_run',
                                 key='no_cleanup',
                                 help_msg=help_msg,
                                 default=False,
                                 key_type=bool,
                                 parser=parser,
                                 long_arg='--dry-run-no-cleanup')

        help_msg = ('Forces the use of a particular job ID. Used internally '
                    'when interacting with an avocado server. You should not '
                    'use this option unless you know exactly what you\'re '
                    'doing')
        settings.register_option(section='run',
                                 key='unique_job_id',
                                 default=None,
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--force-job-id')

        help_msg = 'Forces to use of an alternate job results directory.'
        settings.register_option(section='run',
                                 key='results_dir',
                                 default=None,
                                 metavar='DIRECTORY',
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--job-results-dir')

        help_msg = ('Categorizes this within a directory with the same name, '
                    'by creating a link to the job result directory')
        settings.register_option(section='run',
                                 key='job_category',
                                 help_msg=help_msg,
                                 parser=parser,
                                 default=None,
                                 metavar='CATEGORY',
                                 long_arg='--job-category')

        settings.add_argparser_to_option(namespace='job.run.timeout',
                                         metavar='SECONDS',
                                         parser=parser,
                                         long_arg='--job-timeout')

        help_msg = 'Enable the job interruption on first failed test.'
        settings.register_option(section='run',
                                 key='failfast',
                                 default=False,
                                 key_type=bool,
                                 action='store_true',
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--failfast')

        help_msg = 'Keep job temporary files (useful for avocado debugging).'
        settings.register_option(section='run',
                                 key='keep_tmp',
                                 default=False,
                                 key_type=bool,
                                 action='store_true',
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--keep-tmp')

        help_msg = ('Force the job execution, even if some of the test '
                    'references are not resolved to tests. "on" and '
                    '"off" will be deprecated soon.')
        settings.register_option(section='run',
                                 key='ignore_missing_references',
                                 default=False,
                                 key_type=bool,
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--ignore-missing-references')

        settings.add_argparser_to_option(namespace='sysinfo.collect.enabled',
                                         parser=parser,
                                         action='store_false',
                                         long_arg='--disable-sysinfo')

        settings.add_argparser_to_option('run.execution_order',
                                         parser=parser,
                                         long_arg='--execution-order')

        parser.output = parser.add_argument_group('output and result format')

        help_msg = ('Store given logging STREAMs in '
                    '"$JOB_RESULTS_DIR/$STREAM.$LEVEL."')
        settings.register_option(section='run',
                                 key='store_logging_stream',
                                 nargs='+',
                                 help_msg=help_msg,
                                 default=[],
                                 metavar='STREAM[:LEVEL]',
                                 key_type=list,
                                 parser=parser,
                                 long_arg='--store-logging-stream')

        help_msg = ('Logs the possible data directories for each test. This '
                    'is helpful when writing new tests and not being sure '
                    'where to put data files. Look for "Test data '
                    'directories" in your test log')
        settings.register_option(section='run',
                                 key='log_test_data_directories',
                                 default=False,
                                 key_type=bool,
                                 help_msg=help_msg,
                                 parser=parser,
                                 long_arg='--log-test-data-directories')

        out_check = parser.add_argument_group('output check arguments')

        help_msg = ('Record the output produced by each test (from stdout '
                    'and stderr) into both the current executing result '
                    'and into reference files. Reference files are used on '
                    'subsequent runs to determine if the test produced the '
                    'expected output or not, and the current executing result '
                    'is used to check against a previously recorded reference '
                    'file.  Valid values: "none" (to explicitly disable all '
                    'recording) "stdout" (to record standard output *only*), '
                    '"stderr" (to record standard error *only*), "both" (to '
                    'record standard output and error in separate files), '
                    '"combined" (for standard output and error in a single '
                    'file). "all" is also a valid but deprecated option that '
                    'is a synonym of "both".')
        settings.register_option(section='run',
                                 key='output_check_record',
                                 help_msg=help_msg,
                                 choices=('none', 'stdout', 'stderr',
                                          'both', 'combined', 'all'),
                                 parser=out_check,
                                 default=None,
                                 long_arg='--output-check-record')

        help_msg = ('Disables test output (stdout/stderr) check. If this '
                    'option is given, no output will be checked, even if '
                    'there are reference files present for the test.')
        settings.register_option(section='run',
                                 key='output_check',
                                 default=True,
                                 key_type=bool,
                                 action='store_false',
                                 help_msg=help_msg,
                                 parser=out_check,
                                 long_arg='--disable-output-check')

        loader.add_loader_options(parser, 'run')
        parser_common_args.add_tag_filter_args(parser)

    def run(self, config):
        """
        Run test modules or simple tests.

        :param config: Configuration received from command line parser and
                       possibly other sources.
        :type config: dict
        """
        if 'run.output_check_record' in config:
            check_record = config.get('run.output_check_record')
            process.OUTPUT_CHECK_RECORD_MODE = check_record

        unique_job_id = config.get('run.unique_job_id')
        if unique_job_id is not None:
            try:
                int(unique_job_id, 16)
                if len(unique_job_id) != 40:
                    raise ValueError
            except ValueError:
                LOG_UI.error('Unique Job ID needs to be a 40 digit hex number')
                sys.exit(exit_codes.AVOCADO_FAIL)

        try:
            suite = TestSuite.from_config(config, name='')
            if suite.size == 0:
                sys.exit(exit_codes.AVOCADO_JOB_FAIL)
        except TestSuiteError as err:
            LOG_UI.error(err)
            sys.exit(exit_codes.AVOCADO_JOB_FAIL)
        with job.Job(config, [suite]) as job_instance:
            return job_instance.run()



import json
import os
import unittest

from avocado.core import exit_codes
from avocado.utils import genio, process, script

from .. import AVOCADO, TestCaseTmpDir

AVOCADO_TEST_SKIP_DECORATORS = """
import avocado
from lib_skip_decorators import check_condition

class AvocadoSkipTests(avocado.Test):

    def setUp(self):
        self.log.info('setup executed')

    @avocado.skip('Test skipped')
    def test1(self):
        self.log.info('test executed')

    @avocado.skipIf(check_condition(True),
                    'Skipped due to the True condition')
    def test2(self):
        self.log.info('test executed')

    @avocado.skipUnless(check_condition(False),
                        'Skipped due to the False condition')
    def test3(self):
        self.log.info('test executed')

    def tearDown(self):
        self.log.info('teardown executed')
"""

AVOCADO_TEST_SKIP_CLASS_DECORATORS = """
import avocado
from lib_skip_decorators import check_condition
@avocado.skip('Test skipped')
class AvocadoSkipTests(avocado.Test):

    def setUp(self):
        self.log.info('setup executed')

    def test1(self):
        self.log.info('test executed')

    def test2(self):
        self.log.info('test executed')

    def test3(self):
        self.log.info('test executed')

    def tearDown(self):
        self.log.info('teardown executed')

"""


AVOCADO_TEST_SKIP_IF_CLASS_DECORATORS = """
import avocado
from lib_skip_decorators import check_condition
@avocado.skipIf(check_condition(True),
                    'Skipped due to the True condition')
class AvocadoSkipTests(avocado.Test):

    def setUp(self):
        self.log.info('setup executed')

    def test1(self):
        self.log.info('test executed')

    def test2(self):
        self.log.info('test executed')

    def test3(self):
        self.log.info('test executed')

    def tearDown(self):
        self.log.info('teardown executed')

@avocado.skipIf(check_condition(False),
                    'Skipped due to the True condition')
class AvocadoNoSkipTests(avocado.Test):

    def setUp(self):
        self.log.info('setup executed')

    def test1(self):
        self.log.info('test executed')

    def test2(self):
        self.log.info('test executed')

    def test3(self):
        self.log.info('test executed')

    def tearDown(self):
        self.log.info('teardown executed')
"""


AVOCADO_TEST_SKIP_UNLESS_CLASS_DECORATORS = """
import avocado
from lib_skip_decorators import check_condition

@avocado.skipUnless(check_condition(False),
                    'Skipped due to the True condition')
class AvocadoSkipTests(avocado.Test):

    def setUp(self):
        self.log.info('setup executed')

    def test1(self):
        self.log.info('test executed')

    def test2(self):
        self.log.info('test executed')

    def test3(self):
        self.log.info('test executed')

    def tearDown(self):
        self.log.info('teardown executed')

@avocado.skipUnless(check_condition(True),
                    'Skipped due to the True condition')
class AvocadoNoSkipTests(avocado.Test):

    def setUp(self):
        self.log.info('setup executed')

    def test1(self):
        self.log.info('test executed')

    def test2(self):
        self.log.info('test executed')

    def test3(self):
        self.log.info('test executed')

    def tearDown(self):
        self.log.info('teardown executed')
"""


AVOCADO_TEST_SKIP_LIB = """
def check_condition(condition):
    if condition:
        return True
    return False
"""


AVOCADO_SKIP_DECORATOR_SETUP = """
import avocado

class AvocadoSkipTests(avocado.Test):

    @avocado.skip('Test skipped')
    def setUp(self):
        pass

    def test1(self):
        pass
"""


AVOCADO_SKIP_DECORATOR_TEARDOWN = """
import avocado

class AvocadoSkipTests(avocado.Test):

    def test1(self):
        pass

    @avocado.skip('Test skipped')
    def tearDown(self):
        pass
"""


class TestSkipDecorators(TestCaseTmpDir):

    def setUp(self):
        super(TestSkipDecorators, self).setUp()
        test_path = os.path.join(self.tmpdir.name, 'test_skip_decorators.py')
        self.test_module = script.Script(test_path,
                                         AVOCADO_TEST_SKIP_DECORATORS)
        self.test_module.save()

        class_path = os.path.join(self.tmpdir.name,
                                  'test_skip_class_decorators.py')
        self.class_module = script.Script(class_path,
                                          AVOCADO_TEST_SKIP_CLASS_DECORATORS)
        self.class_module.save()

        class_if_path = os.path.join(self.tmpdir.name,
                                     'test_skip_if_class_decorators.py')
        self.class_if_module = script.Script(class_if_path,
                                             AVOCADO_TEST_SKIP_IF_CLASS_DECORATORS)
        self.class_if_module.save()

        class_unless_path = os.path.join(self.tmpdir.name,
                                         'test_skip_unless_class_decorators.py')
        self.class_unless_module = script.Script(class_unless_path,
                                                 AVOCADO_TEST_SKIP_UNLESS_CLASS_DECORATORS)
        self.class_unless_module.save()

        lib_path = os.path.join(self.tmpdir.name, 'lib_skip_decorators.py')
        self.test_lib = script.Script(lib_path, AVOCADO_TEST_SKIP_LIB)
        self.test_lib.save()

        skip_setup_path = os.path.join(self.tmpdir.name,
                                       'test_skip_decorator_setup.py')
        self.skip_setup = script.Script(skip_setup_path,
                                        AVOCADO_SKIP_DECORATOR_SETUP)
        self.skip_setup.save()

        bad_teardown_path = os.path.join(self.tmpdir.name,
                                         'test_skip_decorator_teardown.py')
        self.bad_teardown = script.Script(bad_teardown_path,
                                          AVOCADO_SKIP_DECORATOR_TEARDOWN)
        self.bad_teardown.save()

    def test_skip_decorators(self):
        cmd_line = [AVOCADO,
                    'run',
                    '--disable-sysinfo',
                    '--job-results-dir',
                    '%s' % self.tmpdir.name,
                    '%s' % self.test_module,
                    '--json -']
        result = process.run(' '.join(cmd_line), ignore_status=True)
        json_results = json.loads(result.stdout_text)
        debuglog = json_results['debuglog']

        self.assertEqual(result.exit_status, exit_codes.AVOCADO_ALL_OK)
        self.assertEqual(json_results['skip'], 3)
        debuglog_contents = genio.read_file(debuglog)
        self.assertFalse('setup executed' in debuglog_contents)
        self.assertFalse('test executed' in debuglog_contents)
        self.assertFalse('teardown executed' in debuglog_contents)

    def test_skip_class_decorators(self):
        cmd_line = [AVOCADO,
                    'run',
                    '--disable-sysinfo',
                    '--job-results-dir',
                    '%s' % self.tmpdir.name,
                    '%s' % self.class_module,
                    '--json -']
        result = process.run(' '.join(cmd_line), ignore_status=True)
        json_results = json.loads(result.stdout_text)
        debuglog = json_results['debuglog']

        self.assertEqual(result.exit_status, exit_codes.AVOCADO_ALL_OK)
        self.assertEqual(json_results['skip'], 3)
        debuglog_contents = genio.read_file(debuglog)
        self.assertFalse('setup executed' in debuglog_contents)
        self.assertFalse('test executed' in debuglog_contents)
        self.assertFalse('teardown executed' in debuglog_contents)

    def test_skipIf_class_decorators(self):
        cmd_line = [AVOCADO,
                    'run',
                    '--disable-sysinfo',
                    '--job-results-dir',
                    '%s' % self.tmpdir.name,
                    '%s' % self.class_if_module,
                    '--json -']
        result = process.run(' '.join(cmd_line), ignore_status=True)
        json_results = json.loads(result.stdout_text)

        self.assertEqual(result.exit_status, exit_codes.AVOCADO_ALL_OK)
        self.assertEqual(json_results['skip'], 3)
        self.assertEqual(json_results['pass'], 3)

    def test_skipUnless_class_decorators(self):
        cmd_line = [AVOCADO,
                    'run',
                    '--disable-sysinfo',
                    '--job-results-dir',
                    '%s' % self.tmpdir.name,
                    '%s' % self.class_unless_module,
                    '--json -']
        result = process.run(' '.join(cmd_line), ignore_status=True)
        json_results = json.loads(result.stdout_text)

        self.assertEqual(result.exit_status, exit_codes.AVOCADO_ALL_OK)
        self.assertEqual(json_results['skip'], 3)
        self.assertEqual(json_results['pass'], 3)

    def test_skip_setup(self):
        cmd_line = [AVOCADO,
                    'run',
                    '--disable-sysinfo',
                    '--job-results-dir',
                    '%s' % self.tmpdir.name,
                    '%s' % self.skip_setup,
                    '--json -']
        result = process.run(' '.join(cmd_line), ignore_status=True)
        json_results = json.loads(result.stdout_text)
        self.assertEqual(result.exit_status, exit_codes.AVOCADO_ALL_OK)
        self.assertEqual(json_results['skip'], 1)

    def test_skip_teardown(self):
        cmd_line = [AVOCADO,
                    'run',
                    '--disable-sysinfo',
                    '--job-results-dir',
                    '%s' % self.tmpdir.name,
                    '%s' % self.bad_teardown,
                    '--json -']
        result = process.run(' '.join(cmd_line), ignore_status=True)
        json_results = json.loads(result.stdout_text)
        self.assertEqual(result.exit_status, exit_codes.AVOCADO_TESTS_FAIL)
        self.assertEqual(json_results['errors'], 1)


if __name__ == '__main__':
    unittest.main()



import os

from avocado.core.output import LOG_UI
from avocado.core.plugin_interfaces import Init, JobPost, JobPre
from avocado.core.settings import settings
from avocado.core.utils import prepend_base_path
from avocado.utils import process


class JobScriptsInit(Init):
    name = 'jobscripts-init'
    description = 'Jobscripts plugin initialization'

    def initialize(self):
        help_msg = 'Warn if configured (or default) directory does not exist'
        settings.register_option(section='plugins.jobscripts',
                                 key='warn_non_existing_dir',
                                 key_type=bool,
                                 default=False,
                                 help_msg=help_msg)

        help_msg = 'Warn if any script run return non-zero status'
        settings.register_option(section='plugins.jobscripts',
                                 key='warn_non_zero_status',
                                 key_type=bool,
                                 default=True,
                                 help_msg=help_msg)

        help_msg = 'Directory with scripts to be executed before a job is run'
        default = '/etc/avocado/scripts/job/pre.d/'
        settings.register_option(section='plugins.jobscripts',
                                 key='pre',
                                 key_type=prepend_base_path,
                                 help_msg=help_msg,
                                 default=default)

        help_msg = 'Directory with scripts to be executed after a job is run'
        default = '/etc/avocado/scripts/job/post.d/'
        settings.register_option(section='plugins.jobscripts',
                                 key='post',
                                 key_type=prepend_base_path,
                                 help_msg=help_msg,
                                 default=default)


class JobScripts(JobPre, JobPost):

    name = 'jobscripts'
    description = 'Runs scripts before/after the job is run'

    def _run_scripts(self, kind, scripts_dir, job):
        config = job.config
        if not os.path.isdir(scripts_dir):
            if config.get('plugins.jobscripts.warn_non_existing_dir'):
                LOG_UI.error("Directory configured to hold %s-job scripts "
                             "has not been found: %s", kind, scripts_dir)
            return

        dir_list = os.listdir(scripts_dir)
        scripts = [os.path.join(scripts_dir, f) for f in dir_list]
        scripts = [f for f in scripts
                   if os.access(f, os.R_OK | os.X_OK)]
        scripts.sort()
        if not scripts:
            return

        env = self._job_to_environment_variables(job)
        non_zero_namespace = 'plugins.jobscripts.warn_non_zero_status'
        warn_non_zero_status = config.get(non_zero_namespace)
        for script in scripts:
            result = process.run(script, ignore_status=True, env=env)
            if (result.exit_status != 0) and warn_non_zero_status:
                LOG_UI.error('%s job script "%s" exited with status "%i"',
                             kind.capitalize(), script, result.exit_status)

    @staticmethod
    def _job_to_environment_variables(job):
        env = {}
        env['AVOCADO_JOB_UNIQUE_ID'] = job.unique_id
        env['AVOCADO_JOB_STATUS'] = job.status
        if job.logdir is not None:
            env['AVOCADO_JOB_LOGDIR'] = job.logdir
        return env

    def pre(self, job):
        path = job.config.get('plugins.jobscripts.pre')
        self._run_scripts('pre', path, job)

    def post(self, job):
        path = job.config.get('plugins.jobscripts.post')
        self._run_scripts('post', path, job)



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2019
# Author: Cleber Rosa <crosa@redhat.com>

"""
data drainer

This module provides utility classes for draining data and dispatching
it to different destinations.  This is intended to be used
concurrently with other code, usually test code producing the output
to be drained/processed.  A thread is started and maintained on behalf
of the user.
"""

import abc
import io
import os
import select
import threading


class BaseDrainer(metaclass=abc.ABCMeta):

    """
    Base drainer, doesn't provide complete functionality to be useful.
    """

    name = 'avocado.utils.datadrainer.BaseDrainer'

    def __init__(self, source, stop_check=None, name=None):
        """
        :param source: where to read data from, this is intentionally
                       abstract
        :param stop_check: callable that should determine if the
                           drainer should quit.  If None is given, it
                           will never stop.
        :type stop_check: function
        :param name: instance name of the drainer, used for describing
                     the name of the thread maintained by this instance
        :type name: str
        """
        self._source = source
        if stop_check is None:
            def stop_check():   # pylint: disable=E0102
                return False
        self._stop_check = stop_check
        if name is not None:
            self.name = name
        # internal state flag, used to stop processing because of a
        # condition that may have happened in between the loop cycles
        self._internal_quit = False

    def data_available(self):
        """
        Checks if source appears to have data to be drained
        """
        return False

    @abc.abstractmethod
    def read(self):
        """
        Abstract method supposed to read from the data source
        """

    @abc.abstractmethod
    def write(self, data):
        """
        Abstract method supposed to write the read data to its destination
        """

    def _loop(self):
        """
        Basic implementation of the thread target

        This loops until either an internal quit flag is set, or when
        the stop_check function evaluates to True.
        """
        while True:
            if self._internal_quit:
                break
            if self.data_available():
                self.write(self.read())
            if self._stop_check():
                break

    def start(self):
        """
        Starts a thread to do the data draining
        """
        self._thread = threading.Thread(target=self._loop,  # pylint: disable=W0201
                                        name=self.name)

        self._thread.daemon = True
        self._thread.start()

    def wait(self):
        """
        Waits on the thread completion
        """
        self._thread.join()


class FDDrainer(BaseDrainer):
    """
    Drainer whose source is a file descriptor

    This drainer uses select to efficiently wait for data to be available on
    a file descriptor.  If the file descriptor is closed, the drainer responds
    by shutting itself down.

    This drainer doesn't provide a write() implementation, and is
    consequently not a complete implementation users can pick and use.
    """

    name = 'avocado.utils.datadrainer.FDDrainer'

    def data_available(self):
        try:
            return select.select([self._source], [], [], 1)[0]
        except OSError as exc:
            if exc.errno == 9:
                return False

    def read(self):
        data = b''
        try:
            data = os.read(self._source, io.DEFAULT_BUFFER_SIZE)
        except OSError as exc:
            if exc.errno == 9:
                self._internal_quit = True
        return data

    def write(self, data):
        # necessary to avoid pylint W0223
        raise NotImplementedError


class BufferFDDrainer(FDDrainer):
    """
    Drains data from a file descriptor and stores it in an internal buffer
    """

    name = 'avocado.utils.datadrainer.BufferFDDrainer'

    def __init__(self, source, stop_check=None, name=None):
        super(BufferFDDrainer, self).__init__(source, stop_check, name)
        self._data = io.BytesIO()

    def write(self, data):
        self._data.write(data)

    @property
    def data(self):
        """
        Returns the buffer data, as bytes
        """
        return self._data.getvalue()


class LineLogger(FDDrainer):

    name = 'avocado.utils.datadrainer.LineLogger'

    def __init__(self, source, stop_check=None, name=None, logger=None):
        super(LineLogger, self).__init__(source, stop_check, name)
        self._logger = logger
        self._buffer = io.BytesIO()

    def write(self, data):
        if b'\n' not in data:
            self._buffer.write(data)
            return
        data = self._buffer.getvalue() + data
        lines = data.split(b'\n')
        if not lines[-1].endswith(b'\n'):
            self._buffer.close()
            self._buffer = io.BytesIO()
            self._buffer.write(lines[-1])
        for line in lines:
            line = line.decode(errors='replace').rstrip('\n')
            if line:
                self._logger.debug(line)



from .. import process


# Probably this will be replaced by aexpect
def run_command(command, host, sudo=False):
    # This is used to avoid circular imports
    if host.__class__.__name__ == 'LocalHost':
        return process.system_output(command, sudo=sudo).decode('utf-8')

    if sudo:
        command = "sudo {}".format(command)
    return host.remote_session.cmd(command).stdout.decode('utf-8')



#!/usr/bin/env python

"""
Script that downloads cloud images via avocado.utils.vmimage
"""

from avocado.core import data_dir
from avocado.utils import vmimage

KNOWN_IMAGES = (
    # from https://download.cirros-cloud.net/0.4.0/MD5SUMS
    ('cirros', '0.4.0', 'aarch64', 'ecc9a5132e7a0f11a4c585f513cd0873', 'md5'),
    ('cirros', '0.4.0', 'arm', '7e9cfcb763e83573a4b9d9315f56cc5f', 'md5'),
    ('cirros', '0.4.0', 'i386', 'b7d8ac291c698c3f1dc0705ce52a3b64', 'md5'),
    ('cirros', '0.4.0', 'x86_64', '443b7623e27ecf03dc9e01ee93f67afe', 'md5'),
)


def main():
    for image in KNOWN_IMAGES:
        name, version, arch, checksum, algorithm = image
        print("%s version %s (%s): " % (name, version, arch), end='')
        download = vmimage.get(name=name, version=version, arch=arch,
                               checksum=checksum, algorithm=algorithm,
                               cache_dir=data_dir.get_cache_dirs()[0])
        print(download.base_image)


if __name__ == '__main__':
    main()



import os
import unittest.mock

from avocado.utils import path


class Path(unittest.TestCase):

    def test_check_readable_exists(self):
        with unittest.mock.patch('avocado.utils.path.os.path.exists',
                                 return_value=False) as mocked_exists:
            with self.assertRaises(OSError) as cm:
                path.check_readable(os.devnull)
            self.assertEqual('File "%s" does not exist' % os.devnull,
                             cm.exception.args[0])
            mocked_exists.assert_called_with(os.devnull)

    def test_check_readable_access(self):
        with unittest.mock.patch('avocado.utils.path.os.access',
                                 return_value=False) as mocked_access:
            with self.assertRaises(OSError) as cm:
                path.check_readable(os.devnull)
            self.assertEqual('File "%s" can not be read' % os.devnull,
                             cm.exception.args[0])
            mocked_access.assert_called_with(os.devnull, os.R_OK)


if __name__ == '__main__':
    unittest.main()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2013-2014
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>


__all__ = ['Test',
           'VERSION',
           'fail_on',
           'cancel_on',
           'skip',
           'skipIf',
           'skipUnless',
           'TestError',
           'TestFail',
           'TestCancel']


from avocado.core import register_core_options, initialize_plugins
from avocado.core.settings import settings

register_core_options()
settings.merge_with_configs()
initialize_plugins()
settings.merge_with_configs()

from avocado.core.decorators import (cancel_on, fail_on, skip, skipIf,
                                     skipUnless)
from avocado.core.exceptions import TestCancel, TestError, TestFail
from avocado.core.test import Test
from avocado.core.version import VERSION
